##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2019 Shirow Miura <shirowmiura@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
Signature analysis function for troubleshooting logic circuits.
This generates the same signature as Hewlett-Packard 5004A.
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Stefan Brüns <stefan.bruens@rwth-aachen.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
This decoder is a simple edge counter.

It can count rising and/or falling edges, provides an optional reset
signal. It can also divide the count to e.g. count the number of
fixed-length words (where a word corresponds to e.g. 9 clock edges).
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2015 Karl Palsson <karlp@tweak.net.au>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

sregs = {
    0: 'RXMCR',
    1: 'PANIDL',
    2: 'PANIDH',
    3: 'SADRL',
    4: 'SADRH',
    5: 'EADR0',
    6: 'EADR1',
    7: 'EADR2',
    8: 'EADR3',
    9: 'EADR4',
    0xa: 'EADR5',
    0xb: 'EADR6',
    0xc: 'EADR7',
    0xd: 'RXFLUSH',
    0xe: 'Reserved',
    0xf: 'Reserved',
    0x10: 'ORDER',
    0x11: 'TXMCR',
    0x12: 'ACKTMOUT',
    0x13: 'ESLOTG1',
    0x14: 'SYMTICKL',
    0x15: 'SYMTICKH',
    0x16: 'PACON0',
    0x17: 'PACON1',
    0x18: 'PACON2',
    0x19: 'Reserved',
    0x1a: 'TXBCON0',
    0x1b: 'TXNCON',
    0x1c: 'TXG1CON',
    0x1d: 'TXG2CON',
    0x1e: 'ESLOTG23',
    0x1f: 'ESLOTG45',
    0x20: 'ESLOTG67',
    0x21: 'TXPEND',
    0x22: 'WAKECON',
    0x23: 'FRMOFFSET',
    0x24: 'TXSTAT',
    0x25: 'TXBCON1',
    0x26: 'GATECLK',
    0x27: 'TXTIME',
    0x28: 'HSYMTIMRL',
    0x29: 'HSYMTIMRH',
    0x2a: 'SOFTRST',
    0x2b: 'Reserved',
    0x2c: 'SECCON0',
    0x2d: 'SECCON1',
    0x2e: 'TXSTBL',
    0x2f: 'Reserved',
    0x30: 'RXSR',
    0x31: 'INTSTAT',
    0x32: 'INTCON',
    0x33: 'GPIO',
    0x34: 'TRISGPIO',
    0x35: 'SLPACK',
    0x36: 'RFCTL',
    0x37: 'SECCR2',
    0x38: 'BBREG0',
    0x39: 'BBREG1',
    0x3a: 'BBREG2',
    0x3b: 'BBREG3',
    0x3c: 'BBREG4',
    0x3d: 'Reserved',
    0x3e: 'BBREG6',
    0x3f: 'CCAEDTH',
}

lregs = {
    0x200: 'RFCON0',
    0x201: 'RFCON1',
    0x202: 'RFCON2',
    0x203: 'RFCON3',
    0x204: 'Reserved',
    0x205: 'RFCON5',
    0x206: 'RFCON6',
    0x207: 'RFCON7',
    0x208: 'RFCON8',
    0x209: 'SLPCAL0',
    0x20A: 'SLPCAL1',
    0x20B: 'SLPCAL2',
    0x20C: 'Reserved',
    0x20D: 'Reserved',
    0x20E: 'Reserved',
    0x20F: 'RFSTATE',
    0x210: 'RSSI',
    0x211: 'SLPCON0',
    0x212: 'Reserved',
    0x213: 'Reserved',
    0x214: 'Reserved',
    0x215: 'Reserved',
    0x216: 'Reserved',
    0x217: 'Reserved',
    0x218: 'Reserved',
    0x219: 'Reserved',
    0x21A: 'Reserved',
    0x21B: 'Reserved',
    0x21C: 'Reserved',
    0x21D: 'Reserved',
    0x21E: 'Reserved',
    0x21F: 'Reserved',
    0x220: 'SLPCON1',
    0x221: 'Reserved',
    0x222: 'WAKETIMEL',
    0x223: 'WAKETIMEH',
    0x224: 'REMCNTL',
    0x225: 'REMCNTH',
    0x226: 'MAINCNT0',
    0x227: 'MAINCNT1',
    0x228: 'MAINCNT2',
    0x229: 'MAINCNT3',
    0x22A: 'Reserved',
    0x22B: 'Reserved',
    0x22C: 'Reserved',
    0x22D: 'Reserved',
    0x22E: 'Reserved',
    0x22F: 'TESTMODE',
    0x230: 'ASSOEADR0',
    0x231: 'ASSOEADR1',
    0x232: 'ASSOEADR2',
    0x233: 'ASSOEADR3',
    0x234: 'ASSOEADR4',
    0x235: 'ASSOEADR5',
    0x236: 'ASSOEADR6',
    0x237: 'ASSOEADR7',
    0x238: 'ASSOSADR0',
    0x239: 'ASSOSADR1',
    0x23A: 'Reserved',
    0x23B: 'Reserved',
    0x23C: 'Unimplemented',
    0x23D: 'Unimplemented',
    0x23E: 'Unimplemented',
    0x23F: 'Unimplemented',
    0x240: 'UPNONCE0',
    0x241: 'UPNONCE1',
    0x242: 'UPNONCE2',
    0x243: 'UPNONCE3',
    0x244: 'UPNONCE4',
    0x245: 'UPNONCE5',
    0x246: 'UPNONCE6',
    0x247: 'UPNONCE7',
    0x248: 'UPNONCE8',
    0x249: 'UPNONCE9',
    0x24A: 'UPNONCE10',
    0x24B: 'UPNONCE11',
    0x24C: 'UPNONCE12'
}



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Steve R <steversig@virginmedia.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
import math
from .lists import *

class Decoder(srd.Decoder):
    api_version = 3
    id = 'ook_oregon'
    name = 'Oregon'
    longname = 'Oregon Scientific'
    desc = 'Oregon Scientific weather sensor protocol.'
    license = 'gplv2+'
    inputs = ['ook']
    outputs = []
    tags = ['Sensor']
    annotations = (
        ('bit', 'Bit'),
        ('field', 'Field'),
        ('l2', 'Level 2'),
        ('pre', 'Preamble'),
        ('syn', 'Sync'),
        ('id', 'SensorID'),
        ('ch', 'Channel'),
        ('roll', 'Rolling code'),
        ('f1', 'Flags1'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0,)),
        ('fields', 'Fields', (1, 3, 4)),
        ('l2', 'Level 2', (2,)),
    )
    binary = (
        ('data-hex', 'Hex data'),
    )
    options = (
        {'id': 'unknown', 'desc': 'Unknown type is', 'default': 'Unknown',
         'values': ('Unknown', 'Temp', 'Temp_Hum', 'Temp_Hum1', 'Temp_Hum_Baro',
                    'Temp_Hum_Baro1', 'UV', 'UV1', 'Wind', 'Rain', 'Rain1')},
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.decoded = [] # Local cache of decoded OOK.
        self.skip = None

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_binary = self.register(srd.OUTPUT_BINARY)
        self.unknown = self.options['unknown']

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    def dump_oregon_hex(self, start, finish):
        nib = self.decoded_nibbles
        hexstring = ''
        for x in nib:
            hexstring += str(x[3]) if x[3] != '' else ' '
        s = 'Oregon ' + self.ver + ' \"' + hexstring.upper() + '\"\n'
        self.put(start, finish, self.out_binary,
                [0, bytes([ord(c) for c in s])])

    def oregon_put_pre_and_sync(self, len_pream, len_sync, ver):
        ook = self.decoded
        self.decode_pos = len_pream
        self.ss, self.es = ook[0][0], ook[self.decode_pos][0]
        self.putx([1, ['Oregon ' + ver + ' Preamble', ver + ' Preamble',
                        ver + ' Pre', ver]])
        self.decode_pos += len_sync
        self.ss, self.es = ook[len_pream][0], ook[self.decode_pos][0]
        self.putx([1, ['Sync', 'Syn', 'S']])

        # Strip off preamble and sync bits.
        self.decoded = self.decoded[self.decode_pos:]
        self.ookstring = self.ookstring[self.decode_pos:]
        self.ver = ver

    def oregon(self):
        self.ookstring = ''
        self.decode_pos = 0
        ook = self.decoded
        for i in range(len(ook)):
            self.ookstring += ook[i][2]
        if '10011001' in self.ookstring[:40]:
            (preamble, data) = self.ookstring.split('10011001', 1)
            if len(data) > 0 and len(preamble) > 16:
                self.oregon_put_pre_and_sync(len(preamble), 8, 'v2.1')
                self.oregon_v2()
        elif 'E1100' in self.ookstring[:17]:
            (preamble, data) = self.ookstring.split('E1100', 1)
            if len(data) > 0 and len(preamble) <= 12:
                self.oregon_put_pre_and_sync(len(preamble), 5, 'v1')
                self.oregon_v1()
        elif '0101' in self.ookstring[:28]:
            (preamble, data) = self.ookstring.split('0101', 1)
            if len(data) > 0 and len(preamble) > 12:
                self.oregon_put_pre_and_sync(len(preamble), 4, 'v3')
                self.oregon_v3()
        elif len(self.ookstring) > 16: # Ignore short packets.
            error_message = 'Not Oregon or wrong preamble'
            self.ss, self.es = ook[0][0], ook[len(ook) - 1][1]
            self.putx([1,[error_message]])

    def oregon_v1(self):
        ook = self.decoded
        self.decode_pos = 0
        self.decoded_nibbles = []
        if len(self.decoded) >= 32: # Check there are at least 8 nibbles.
            self.oregon_put_nib('RollingCode', ook[self.decode_pos][0],
                                ook[self.decode_pos + 3][1], 4)
            self.oregon_put_nib('Ch', ook[self.decode_pos][0],
                                ook[self.decode_pos + 3][1], 4)
            self.oregon_put_nib('Temp', ook[self.decode_pos][0],
                                ook[self.decode_pos + 15][1], 16)
            self.oregon_put_nib('Checksum', ook[self.decode_pos][0],
                                ook[self.decode_pos + 7][1], 8)

            self.dump_oregon_hex(ook[0][0], ook[len(ook) - 1][1])

            # L2 decode.
            self.oregon_temp(2)
            self.oregon_channel(1)
            self.oregon_battery(2)
            self.oregon_checksum_v1()

    def oregon_v2(self): # Convert to v3 format - discard odd bits.
        self.decode_pos = 0
        self.ookstring = self.ookstring[1::2]
        for i in range(len(self.decoded)):
            if i % 2 == 1:
                self.decoded[i][0] = self.decoded[i - 1][0] # Re-align start pos.
        self.decoded = self.decoded[1::2] # Discard left hand bits.
        self.oregon_v3() # Decode with v3 decoder.

    def oregon_nibbles(self, ookstring):
        num_nibbles = int(len(ookstring) / 4)
        nibbles = []
        for i in range(num_nibbles):
            nibble = ookstring[4 * i : 4 * i + 4]
            nibble = nibble[::-1] # Reversed from right.
            nibbles.append(nibble)
        return nibbles

    def oregon_put_nib(self, label, start, finish, numbits):
        param = self.ookstring[self.decode_pos:self.decode_pos + numbits]
        param = self.oregon_nibbles(param)
        if 'E' in ''.join(param): # Blank out fields with errors.
            result = ''
        else:
            result = hex(int(''.join(param), 2))[2:]
            if len(result) < numbits / 4: # Reinstate leading zeros.
                result = '0' * (int(numbits / 4) - len(result)) + result
        if label != '':
            label += ': '
        self.put(start, finish, self.out_ann, [1, [label + result, result]])
        if label == '': # No label - use nibble position.
            label = int(self.decode_pos / 4)
        for i in range(len(param)):
            ss = self.decoded[self.decode_pos + (4 * i)][0]
            es = self.decoded[self.decode_pos + (4 * i) + 3][1]
            # Blank out nibbles with errors.
            result = '' if ('E' in param[i]) else hex(int(param[i], 2))[2:]
            # Save nibbles for L2 decoder.
            self.decoded_nibbles.append([ss, es, label, result])
        self.decode_pos += numbits

    def oregon_v3(self):
        self.decode_pos = 0
        self.decoded_nibbles = []
        ook = self.decoded

        if len(self.decoded) >= 32: # Check there are at least 8 nibbles.
            self.oregon_put_nib('SensorID', ook[self.decode_pos][0],
                                ook[self.decode_pos + 16][0], 16)
            self.oregon_put_nib('Ch', ook[self.decode_pos][0],
                                ook[self.decode_pos + 3][1], 4)
            self.oregon_put_nib('RollingCode', ook[self.decode_pos][0],
                                ook[self.decode_pos + 7][1], 8)
            self.oregon_put_nib('Flags1', ook[self.decode_pos][0],
                                ook[self.decode_pos + 3][1], 4)

            rem_nibbles = len(self.ookstring[self.decode_pos:]) // 4
            for i in range(rem_nibbles): # Display and save rest of nibbles.
                self.oregon_put_nib('', ook[self.decode_pos][0],
                                    ook[self.decode_pos + 3][1], 4)
            self.dump_oregon_hex(ook[0][0], ook[len(ook) - 1][1])
            self.oregon_level2() # Level 2 decode.
        else:
            error_message = 'Too short to decode'
            self.put(ook[0][0], ook[-1][1], self.out_ann, [1, [error_message]])

    def oregon_put_l2_param(self, offset, digits, dec_point, pre_label, label):
        nib = self.decoded_nibbles
        result = 0
        out_string = ''.join(str(x[3]) for x in nib[offset:offset + digits])
        if len(out_string) == digits:
            for i in range(dec_point, 0, -1):
                result += int(nib[offset + dec_point - i][3], 16) / pow(10, i)
            for i in range(dec_point, digits):
                result += int(nib[offset + i][3], 16) * pow(10, i - dec_point)
            result = '%g' % (result)
        else:
            result = ''
        es = nib[offset + digits - 1][1]
        if label == '\u2103':
            es = nib[offset + digits][1] # Align temp to include +/- nibble.
        self.put(nib[offset][0], es, self.out_ann,
                [2, [pre_label + result + label, result]])

    def oregon_temp(self, offset):
        nib = self.decoded_nibbles
        if nib[offset + 3][3] != '':
            temp_sign = str(int(nib[offset + 3][3], 16))
            temp_sign = '-' if temp_sign != '0' else '+'
        else:
            temp_sign = '?'
        self.oregon_put_l2_param(offset, 3, 1, temp_sign, '\u2103')

    def oregon_baro(self, offset):
        nib = self.decoded_nibbles
        baro = ''
        if not (nib[offset + 2][3] == '' or nib[offset + 1][3] == ''
           or nib[offset][3] == ''):
            baro = str(int(nib[offset + 1][3] + nib[offset][3], 16) + 856)
        self.put(nib[offset][0], nib[offset + 3][1],
                 self.out_ann, [2, [baro + ' mb', baro]])

    def oregon_wind_dir(self, offset):
        nib = self.decoded_nibbles
        if nib[offset][3] != '':
            w_dir = int(int(nib[offset][3], 16) * 22.5)
            w_compass = dir_table[math.floor((w_dir + 11.25) / 22.5)]
            self.put(nib[offset][0], nib[offset][1], self.out_ann,
                [2, [w_compass + ' (' + str(w_dir) + '\u00b0)', w_compass]])

    def oregon_channel(self, offset):
        nib = self.decoded_nibbles
        channel = ''
        if nib[offset][3] != '':
            ch = int(nib[offset][3], 16)
            if self.ver != 'v3': # May not be true for all v2.1 sensors.
                if ch != 0:
                    bit_pos = 0
                    while ((ch & 1) == 0):
                        bit_pos += 1
                        ch = ch >> 1
                    if self.ver == 'v2.1':
                        bit_pos += 1
                    channel = str(bit_pos)
            elif self.ver == 'v3': # Not sure if this applies to all v3's.
                channel = str(ch)
        if channel != '':
            self.put(nib[offset][0], nib[offset][1],
                     self.out_ann, [2, ['Ch ' + channel, channel]])

    def oregon_battery(self, offset):
        nib = self.decoded_nibbles
        batt = 'OK'
        if nib[offset][3] != '':
            if (int(nib[offset][3], 16) >> 2) & 0x1 == 1:
                batt = 'Low'
            self.put(nib[offset][0], nib[offset][1],
                     self.out_ann, [2, ['Batt ' + batt, batt]])

    def oregon_level2(self): # v2 and v3 level 2 decoder.
        nib = self.decoded_nibbles
        self.sensor_id = (nib[0][3] + nib[1][3] + nib[2][3] + nib[3][3]).upper()
        nl, sensor_type = sensor.get(self.sensor_id, [['Unknown'], 'Unknown'])
        names = ','.join(nl)
        # Allow user to try decoding an unknown sensor.
        if sensor_type == 'Unknown' and self.unknown != 'Unknown':
            sensor_type = self.unknown
        self.put(nib[0][0], nib[3][1], self.out_ann,
            [2, [names + ' - ' + sensor_type, names, nl[0]]])
        self.oregon_channel(4)
        self.oregon_battery(7)
        if sensor_type == 'Rain':
            self.oregon_put_l2_param(8, 4, 2, '', ' in/hr')     # Rain rate
            self.oregon_put_l2_param(12, 6, 3, 'Total ', ' in') # Rain total
            self.oregon_checksum(18)
        if sensor_type == 'Rain1':
            self.oregon_put_l2_param(8, 3, 1, '', ' mm/hr')     # Rain rate
            self.oregon_put_l2_param(11, 5, 1, 'Total ', ' mm') # Rain total
            self.oregon_checksum(18)
        if sensor_type == 'Temp':
            self.oregon_temp(8)
            self.oregon_checksum(12)
        if sensor_type == 'Temp_Hum_Baro':
            self.oregon_temp(8)
            self.oregon_put_l2_param(12, 2, 0, 'Hum ', '%') # Hum
            self.oregon_baro(15)                            # Baro
            self.oregon_checksum(19)
        if sensor_type == 'Temp_Hum_Baro1':
            self.oregon_temp(8)
            self.oregon_put_l2_param(12, 2, 0, 'Hum ', '%') # Hum
            self.oregon_baro(14)                            # Baro
        if sensor_type == 'Temp_Hum':
            self.oregon_temp(8)
            self.oregon_put_l2_param(12, 2, 0, 'Hum ', '%') # Hum
            self.oregon_checksum(15)
        if sensor_type == 'Temp_Hum1':
            self.oregon_temp(8)
            self.oregon_put_l2_param(12, 2, 0, 'Hum ', '%') # Hum
            self.oregon_checksum(14)
        if sensor_type == 'UV':
            self.oregon_put_l2_param(8, 2, 0, '', '') # UV
        if sensor_type == 'UV1':
            self.oregon_put_l2_param(11, 2, 0,'' ,'') # UV
        if sensor_type == 'Wind':
            self.oregon_wind_dir(8)
            self.oregon_put_l2_param(11, 3, 1, 'Gust ', ' m/s')  # Wind gust
            self.oregon_put_l2_param(14, 3, 1, 'Speed ', ' m/s') # Wind speed
            self.oregon_checksum(17)

    def oregon_put_checksum(self, nibbles, checksum):
        nib = self.decoded_nibbles
        result = 'BAD'
        if (nibbles + 1) < len(nib):
            if (nib[nibbles + 1][3] != '' and nib[nibbles][3] != ''
                 and checksum != -1):
                if self.ver != 'v1':
                    if checksum == (int(nib[nibbles + 1][3], 16) * 16 +
                                    int(nib[nibbles][3], 16)):
                        result = 'OK'
                else:
                    if checksum == (int(nib[nibbles][3], 16) * 16 +
                                    int(nib[nibbles + 1][3], 16)):
                        result = 'OK'
            rx_check = (nib[nibbles + 1][3] + nib[nibbles][3]).upper()
            details = '%s Calc %s Rx %s ' % (result, hex(checksum)[2:].upper(),
                                             rx_check)
            self.put(nib[nibbles][0], nib[nibbles + 1][1],
                     self.out_ann, [2, ['Checksum ' + details, result]])

    def oregon_checksum(self, nibbles):
        checksum = 0
        for i in range(nibbles):        # Add reversed nibbles.
            nibble = self.ookstring[i * 4 : i * 4 + 4]
            nibble = nibble[::-1]       # Reversed from right.
            if 'E' in nibble:           # Abort checksum if there are errors.
                checksum = -1
                break
            checksum += int(nibble, 2)
            if checksum > 255:
                checksum -= 255         # Make it roll over at 255.
        chk_ver, comment = sensor_checksum.get(self.sensor_id,
                                               ['Unknown', 'Unknown'])
        if chk_ver != 'Unknown':
            self.ver = chk_ver
        if self.ver == 'v2.1':
            checksum -= 10              # Subtract 10 from v2 checksums.
        self.oregon_put_checksum(nibbles, checksum)

    def oregon_checksum_v1(self):
        nib = self.decoded_nibbles
        checksum = 0
        for i in range(3):              # Add the first three bytes.
            if nib[2 * i][3] == '' or nib[2 * i + 1][3] == '': # Abort if blank.
                checksum = -1
                break
            checksum += ((int(nib[2 * i][3], 16) & 0xF) << 4 |
                         (int(nib[2 * i + 1][3], 16) & 0xF))
            if checksum > 255:
                checksum -= 255         # Make it roll over at 255.
        self.oregon_put_checksum(6, checksum)

    def decode(self, ss, es, data):
        self.decoded = data
        self.oregon()



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012-2014 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from .parts import *

VENDOR_CODE_ATMEL = 0x1e

class Decoder(srd.Decoder):
    api_version = 3
    id = 'avr_isp'
    name = 'AVR ISP'
    longname = 'AVR In-System Programming'
    desc = 'Atmel AVR In-System Programming (ISP) protocol.'
    license = 'gplv2+'
    inputs = ['spi']
    outputs = []
    tags = ['Debug/trace']
    annotations = (
        ('pe', 'Programming enable'),
        ('rsb0', 'Read signature byte 0'),
        ('rsb1', 'Read signature byte 1'),
        ('rsb2', 'Read signature byte 2'),
        ('ce', 'Chip erase'),
        ('rfb', 'Read fuse bits'),
        ('rhfb', 'Read high fuse bits'),
        ('refb', 'Read extended fuse bits'),
        ('warnings', 'Warnings'),
        ('dev', 'Device'),
    )
    annotation_rows = (
        ('bits', 'Bits', ()),
        ('commands', 'Commands', tuple(range(7 + 1))),
        ('warnings', 'Warnings', (8,)),
        ('dev', 'Device', (9,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'IDLE'
        self.mosi_bytes, self.miso_bytes = [], []
        self.ss_cmd, self.es_cmd = 0, 0
        self.xx, self.yy, self.zz, self.mm = 0, 0, 0, 0
        self.ss_device = None

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss_cmd, self.es_cmd, self.out_ann, data)

    def handle_cmd_programming_enable(self, cmd, ret):
        # Programming enable.
        # Note: The chip doesn't send any ACK for 'Programming enable'.
        self.putx([0, ['Programming enable']])

        # Sanity check on reply.
        if ret[1:4] != [0xac, 0x53, cmd[2]]:
            self.putx([8, ['Warning: Unexpected bytes in reply!']])

    def handle_cmd_read_signature_byte_0x00(self, cmd, ret):
        # Signature byte 0x00: vendor code.
        self.vendor_code = ret[3]
        v = vendor_code[self.vendor_code]
        self.putx([1, ['Vendor code: 0x%02x (%s)' % (ret[3], v)]])

        # Store for later.
        self.xx = cmd[1] # Same as ret[2].
        self.yy = cmd[3]
        self.zz = ret[0]

        # Sanity check on reply.
        if ret[1] != 0x30 or ret[2] != cmd[1]:
            self.putx([8, ['Warning: Unexpected bytes in reply!']])

        # Sanity check for the vendor code.
        if self.vendor_code != VENDOR_CODE_ATMEL:
            self.putx([8, ['Warning: Vendor code was not 0x1e (Atmel)!']])

    def handle_cmd_read_signature_byte_0x01(self, cmd, ret):
        # Signature byte 0x01: part family and memory size.
        self.part_fam_flash_size = ret[3]
        self.putx([2, ['Part family / memory size: 0x%02x' % ret[3]]])

        # Store for later.
        self.mm = cmd[3]
        self.ss_device = self.ss_cmd

        # Sanity check on reply.
        if ret[1] != 0x30 or ret[2] != cmd[1] or ret[0] != self.yy:
            self.putx([8, ['Warning: Unexpected bytes in reply!']])

    def handle_cmd_read_signature_byte_0x02(self, cmd, ret):
        # Signature byte 0x02: part number.
        self.part_number = ret[3]
        self.putx([3, ['Part number: 0x%02x' % ret[3]]])

        p = part[(self.part_fam_flash_size, self.part_number)]
        data = [9, ['Device: Atmel %s' % p]]
        self.put(self.ss_device, self.es_cmd, self.out_ann, data)

        # Sanity check on reply.
        if ret[1] != 0x30 or ret[2] != self.xx or ret[0] != self.mm:
            self.putx([8, ['Warning: Unexpected bytes in reply!']])

        self.xx, self.yy, self.zz, self.mm = 0, 0, 0, 0

    def handle_cmd_chip_erase(self, cmd, ret):
        # Chip erase (erases both flash an EEPROM).
        # Upon successful chip erase, the lock bits will also be erased.
        # The only way to end a Chip Erase cycle is to release RESET#.
        self.putx([4, ['Chip erase']])

        # TODO: Check/handle RESET#.

        # Sanity check on reply.
        bit = (ret[2] & (1 << 7)) >> 7
        if ret[1] != 0xac or bit != 1 or ret[3] != cmd[2]:
            self.putx([8, ['Warning: Unexpected bytes in reply!']])

    def handle_cmd_read_fuse_bits(self, cmd, ret):
        # Read fuse bits.
        self.putx([5, ['Read fuse bits: 0x%02x' % ret[3]]])

        # TODO: Decode fuse bits.
        # TODO: Sanity check on reply.

    def handle_cmd_read_fuse_high_bits(self, cmd, ret):
        # Read fuse high bits.
        self.putx([6, ['Read fuse high bits: 0x%02x' % ret[3]]])

        # TODO: Decode fuse bits.
        # TODO: Sanity check on reply.

    def handle_cmd_read_extended_fuse_bits(self, cmd, ret):
        # Read extended fuse bits.
        self.putx([7, ['Read extended fuse bits: 0x%02x' % ret[3]]])

        # TODO: Decode fuse bits.
        # TODO: Sanity check on reply.

    def handle_command(self, cmd, ret):
        if cmd[:2] == [0xac, 0x53]:
            self.handle_cmd_programming_enable(cmd, ret)
        elif cmd[0] == 0xac and (cmd[1] & (1 << 7)) == (1 << 7):
            self.handle_cmd_chip_erase(cmd, ret)
        elif cmd[:3] == [0x50, 0x00, 0x00]:
            self.handle_cmd_read_fuse_bits(cmd, ret)
        elif cmd[:3] == [0x58, 0x08, 0x00]:
            self.handle_cmd_read_fuse_high_bits(cmd, ret)
        elif cmd[:3] == [0x50, 0x08, 0x00]:
            self.handle_cmd_read_extended_fuse_bits(cmd, ret)
        elif cmd[0] == 0x30 and cmd[2] == 0x00:
            self.handle_cmd_read_signature_byte_0x00(cmd, ret)
        elif cmd[0] == 0x30 and cmd[2] == 0x01:
            self.handle_cmd_read_signature_byte_0x01(cmd, ret)
        elif cmd[0] == 0x30 and cmd[2] == 0x02:
            self.handle_cmd_read_signature_byte_0x02(cmd, ret)
        else:
            c = '%02x %02x %02x %02x' % tuple(cmd)
            r = '%02x %02x %02x %02x' % tuple(ret)
            self.putx([0, ['Unknown command: %s (reply: %s)!' % (c, r)]])

    def decode(self, ss, es, data):
        ptype, mosi, miso = data

        # For now, only use DATA and BITS packets.
        if ptype not in ('DATA', 'BITS'):
            return

        # Store the individual bit values and ss/es numbers. The next packet
        # is guaranteed to be a 'DATA' packet belonging to this 'BITS' one.
        if ptype == 'BITS':
            self.miso_bits, self.mosi_bits = miso, mosi
            return

        self.ss, self.es = ss, es

        if len(self.mosi_bytes) == 0:
            self.ss_cmd = ss

        # Append new bytes.
        self.mosi_bytes.append(mosi)
        self.miso_bytes.append(miso)

        # All commands consist of 4 bytes.
        if len(self.mosi_bytes) < 4:
            return

        self.es_cmd = es

        self.handle_command(self.mosi_bytes, self.miso_bytes)

        self.mosi_bytes = []
        self.miso_bytes = []



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2014 Daniel Elstner <daniel.kitta@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
The Zilog Z80 is an 8-bit microprocessor compatible with the Intel 8080.

In addition to the 8-bit data bus, this decoder requires the input signals
/M1 (machine cycle), /RD (read) and /WR (write) to do its work. An explicit
clock signal is not required. However, the Z80 CPU clock may be used as
sampling clock, if applicable.

Notes on the Z80 opcode format and descriptions of both documented and
"undocumented" opcodes are available here:

Details:
http://www.z80.info/decoding.htm
http://clrhome.org/table/
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2010-2014 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class Decoder(srd.Decoder):
    api_version = 3
    id = 'nunchuk'
    name = 'Nunchuk'
    longname = 'Nintendo Wii Nunchuk'
    desc = 'Nintendo Wii Nunchuk controller protocol.'
    license = 'gplv2+'
    inputs = ['i2c']
    outputs = []
    tags = ['Sensor']
    annotations = \
        tuple(('reg-0x%02X' % i, 'Register 0x%02X' % i) for i in range(6)) + (
        ('bit-bz', 'BZ bit'),
        ('bit-bc', 'BC bit'),
        ('bit-ax', 'AX bits'),
        ('bit-ay', 'AY bits'),
        ('bit-az', 'AZ bits'),
        ('nunchuk-write', 'Nunchuk write'),
        ('cmd-init', 'Init command'),
        ('summary', 'Summary'),
        ('warnings', 'Warnings'),
    )
    annotation_rows = (
        ('regs', 'Registers', tuple(range(13))),
        ('summary', 'Summary', (13,)),
        ('warnings', 'Warnings', (14,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'IDLE'
        self.sx = self.sy = self.ax = self.ay = self.az = self.bz = self.bc = -1
        self.databytecount = 0
        self.reg = 0x00
        self.ss = self.es = self.ss_block = self.es_block = 0
        self.init_seq = []

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    def putb(self, data):
        self.put(self.ss_block, self.es_block, self.out_ann, data)

    def putd(self, bit1, bit2, data):
        self.put(self.bits[bit1][1], self.bits[bit2][2], self.out_ann, data)

    def handle_reg_0x00(self, databyte):
        self.ss_block = self.ss
        self.sx = databyte
        self.putx([0, ['Analog stick X position: 0x%02X' % self.sx,
                       'SX: 0x%02X' % self.sx]])

    def handle_reg_0x01(self, databyte):
        self.sy = databyte
        self.putx([1, ['Analog stick Y position: 0x%02X' % self.sy,
                       'SY: 0x%02X' % self.sy]])

    def handle_reg_0x02(self, databyte):
        self.ax = databyte << 2
        self.putx([2, ['Accelerometer X value bits[9:2]: 0x%03X' % self.ax,
                       'AX[9:2]: 0x%03X' % self.ax]])

    def handle_reg_0x03(self, databyte):
        self.ay = databyte << 2
        self.putx([3, ['Accelerometer Y value bits[9:2]: 0x%03X' % self.ay,
                       'AY[9:2]: 0x%03X' % self.ay]])

    def handle_reg_0x04(self, databyte):
        self.az = databyte << 2
        self.putx([4, ['Accelerometer Z value bits[9:2]: 0x%03X' % self.az,
                       'AZ[9:2]: 0x%03X' % self.az]])

    def handle_reg_0x05(self, databyte):
        self.es_block = self.es
        self.bz = (databyte & (1 << 0)) >> 0 # Bits[0:0]
        self.bc = (databyte & (1 << 1)) >> 1 # Bits[1:1]
        ax_rest = (databyte & (3 << 2)) >> 2 # Bits[3:2]
        ay_rest = (databyte & (3 << 4)) >> 4 # Bits[5:4]
        az_rest = (databyte & (3 << 6)) >> 6 # Bits[7:6]
        self.ax |= ax_rest
        self.ay |= ay_rest
        self.az |= az_rest

        # self.putx([5, ['Register 5', 'Reg 5', 'R5']])

        s = '' if (self.bz == 0) else 'not '
        self.putd(0, 0, [6, ['Z: %spressed' % s, 'BZ: %d' % self.bz]])

        s = '' if (self.bc == 0) else 'not '
        self.putd(1, 1, [7, ['C: %spressed' % s, 'BC: %d' % self.bc]])

        self.putd(3, 2, [8, ['Accelerometer X value bits[1:0]: 0x%X' % ax_rest,
                             'AX[1:0]: 0x%X' % ax_rest]])

        self.putd(5, 4, [9, ['Accelerometer Y value bits[1:0]: 0x%X' % ay_rest,
                             'AY[1:0]: 0x%X' % ay_rest]])

        self.putd(7, 6, [10, ['Accelerometer Z value bits[1:0]: 0x%X' % az_rest,
                              'AZ[1:0]: 0x%X' % az_rest]])

        self.reg = 0x00

    def output_full_block_if_possible(self):
        # For now, only output summary annotations if all values are available.
        t = (self.sx, self.sy, self.ax, self.ay, self.az, self.bz, self.bc)
        if -1 in t:
            return
        bz = 'pressed' if self.bz == 0 else 'not pressed'
        bc = 'pressed' if self.bc == 0 else 'not pressed'
        s = 'Analog stick: %d/%d, accelerometer: %d/%d/%d, Z: %s, C: %s' % \
            (self.sx, self.sy, self.ax, self.ay, self.az, bz, bc)
        self.putb([13, [s]])

    def handle_reg_write(self, databyte):
        self.putx([11, ['Nunchuk write: 0x%02X' % databyte]])
        if len(self.init_seq) < 2:
            self.init_seq.append(databyte)

    def output_init_seq(self):
        if len(self.init_seq) != 2:
            self.putb([14, ['Init sequence was %d bytes long (2 expected)' % \
                      len(self.init_seq)]])
            return

        if self.init_seq != [0x40, 0x00]:
            self.putb([14, ['Unknown init sequence (expected: 0x40 0x00)']])
            return

        # TODO: Detect Nunchuk clones (they have different init sequences).

        self.putb([12, ['Initialize Nunchuk', 'Init Nunchuk', 'Init', 'I']])

    def decode(self, ss, es, data):
        cmd, databyte = data

        # Collect the 'BITS' packet, then return. The next packet is
        # guaranteed to belong to these bits we just stored.
        if cmd == 'BITS':
            self.bits = databyte
            return

        self.ss, self.es = ss, es

        # State machine.
        if self.state == 'IDLE':
            # Wait for an I²C START condition.
            if cmd != 'START':
                return
            self.state = 'GET SLAVE ADDR'
            self.ss_block = ss
        elif self.state == 'GET SLAVE ADDR':
            # Wait for an address read/write operation.
            if cmd == 'ADDRESS READ':
                self.state = 'READ REGS'
            elif cmd == 'ADDRESS WRITE':
                self.state = 'WRITE REGS'
        elif self.state == 'READ REGS':
            if cmd == 'DATA READ':
                handle_reg = getattr(self, 'handle_reg_0x%02x' % self.reg)
                handle_reg(databyte)
                self.reg += 1
            elif cmd == 'STOP':
                self.es_block = es
                self.output_full_block_if_possible()
                self.sx = self.sy = self.ax = self.ay = self.az = -1
                self.bz = self.bc = -1
                self.state = 'IDLE'
            else:
                # self.putx([14, ['Ignoring: %s (data=%s)' % (cmd, databyte)]])
                pass
        elif self.state == 'WRITE REGS':
            if cmd == 'DATA WRITE':
                self.handle_reg_write(databyte)
            elif cmd == 'STOP':
                self.es_block = es
                self.output_init_seq()
                self.init_seq = []
                self.state = 'IDLE'
            else:
                # self.putx([14, ['Ignoring: %s (data=%s)' % (cmd, databyte)]])
                pass



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2012-2013 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

# ...
fields = {
    # START field (indicates start or stop of a transaction)
    'START': {
        0b0000: 'Start of cycle for a target',
        0b0001: 'Reserved',
        0b0010: 'Grant for bus master 0',
        0b0011: 'Grant for bus master 1',
        0b0100: 'Reserved',
        0b0101: 'Reserved',
        0b0110: 'Reserved',
        0b0111: 'Reserved',
        0b1000: 'Reserved',
        0b1001: 'Reserved',
        0b1010: 'Reserved',
        0b1011: 'Reserved',
        0b1100: 'Reserved',
        0b1101: 'Start of cycle for a Firmware Memory Read cycle',
        0b1110: 'Start of cycle for a Firmware Memory Write cycle',
        0b1111: 'Stop/abort (end of a cycle for a target)',
    },
    # Cycle type / direction field
    # Bit 0 (LAD[0]) is unused, should always be 0.
    # Neither host nor peripheral are allowed to drive 0b11x0.
    'CT_DR': {
        0b0000: 'I/O read',
        0b0010: 'I/O write',
        0b0100: 'Memory read',
        0b0110: 'Memory write',
        0b1000: 'DMA read',
        0b1010: 'DMA write',
        0b1100: 'Reserved / not allowed',
        0b1110: 'Reserved / not allowed',
    },
    # SIZE field (determines how many bytes are to be transferred)
    # Bits[3:2] are reserved, must be driven to 0b00.
    # Neither host nor peripheral are allowed to drive 0b0010.
    'SIZE': {
        0b0000: '8 bits (1 byte)',
        0b0001: '16 bits (2 bytes)',
        0b0010: 'Reserved / not allowed',
        0b0011: '32 bits (4 bytes)',
    },
    # CHANNEL field (bits[2:0] contain the DMA channel number)
    'CHANNEL': {
        0b0000: '0',
        0b0001: '1',
        0b0010: '2',
        0b0011: '3',
        0b0100: '4',
        0b0101: '5',
        0b0110: '6',
        0b0111: '7',
    },
    # SYNC field (used to add wait states)
    'SYNC': {
        0b0000: 'Ready',
        0b0001: 'Reserved',
        0b0010: 'Reserved',
        0b0011: 'Reserved',
        0b0100: 'Reserved',
        0b0101: 'Short wait',
        0b0110: 'Long wait',
        0b0111: 'Reserved',
        0b1000: 'Reserved',
        0b1001: 'Ready more (DMA only)',
        0b1010: 'Error',
        0b1011: 'Reserved',
        0b1100: 'Reserved',
        0b1101: 'Reserved',
        0b1110: 'Reserved',
        0b1111: 'Reserved',
    },
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'lpc'
    name = 'LPC'
    longname = 'Low Pin Count'
    desc = 'Protocol for low-bandwidth devices on PC mainboards.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['PC']
    channels = (
        {'id': 'lframe', 'name': 'LFRAME#', 'desc': 'Frame'},
        {'id': 'lclk',   'name': 'LCLK',    'desc': 'Clock'},
        {'id': 'lad0',   'name': 'LAD[0]',  'desc': 'Addr/control/data 0'},
        {'id': 'lad1',   'name': 'LAD[1]',  'desc': 'Addr/control/data 1'},
        {'id': 'lad2',   'name': 'LAD[2]',  'desc': 'Addr/control/data 2'},
        {'id': 'lad3',   'name': 'LAD[3]',  'desc': 'Addr/control/data 3'},
    )
    optional_channels = (
        {'id': 'lreset', 'name': 'LRESET#', 'desc': 'Reset'},
        {'id': 'ldrq',   'name': 'LDRQ#',   'desc': 'Encoded DMA / bus master request'},
        {'id': 'serirq', 'name': 'SERIRQ',  'desc': 'Serialized IRQ'},
        {'id': 'clkrun', 'name': 'CLKRUN#', 'desc': 'Clock run'},
        {'id': 'lpme',   'name': 'LPME#',   'desc': 'LPC power management event'},
        {'id': 'lpcpd',  'name': 'LPCPD#',  'desc': 'Power down'},
        {'id': 'lsmi',   'name': 'LSMI#',   'desc': 'System Management Interrupt'},
    )
    annotations = (
        ('warnings', 'Warnings'),
        ('start', 'Start'),
        ('cycle-type', 'Cycle-type/direction'),
        ('addr', 'Address'),
        ('tar1', 'Turn-around cycle 1'),
        ('sync', 'Sync'),
        ('data', 'Data'),
        ('tar2', 'Turn-around cycle 2'),
    )
    annotation_rows = (
        ('data', 'Data', (1, 2, 3, 4, 5, 6, 7)),
        ('warnings', 'Warnings', (0,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'IDLE'
        self.oldlclk = -1
        self.lad = -1
        self.addr = 0
        self.cur_nibble = 0
        self.cycle_type = -1
        self.databyte = 0
        self.tarcount = 0
        self.synccount = 0
        self.oldpins = None
        self.ss_block = self.es_block = None

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putb(self, data):
        self.put(self.ss_block, self.es_block, self.out_ann, data)

    def handle_get_start(self, lad, lad_bits, lframe):
        # LAD[3:0]: START field (1 clock cycle).

        # The last value of LAD[3:0] before LFRAME# gets de-asserted is what
        # the peripherals must use. However, the host can keep LFRAME# asserted
        # multiple clocks, and we output all START fields that occur, even
        # though the peripherals are supposed to ignore all but the last one.
        self.es_block = self.samplenum
        self.putb([1, [fields['START'][lad], 'START', 'St', 'S']])
        self.ss_block = self.samplenum

        # Output a warning if LAD[3:0] changes while LFRAME# is low.
        # TODO
        if (self.lad != -1 and self.lad != lad):
            self.putb([0, ['LAD[3:0] changed while LFRAME# was asserted']])

        # LFRAME# is asserted (low). Wait until it gets de-asserted again
        # (the host is allowed to keep it asserted multiple clocks).
        if lframe != 1:
            return

        self.start_field = self.lad
        self.state = 'GET CT/DR'

    def handle_get_ct_dr(self, lad, lad_bits):
        # LAD[3:0]: Cycle type / direction field (1 clock cycle).

        self.cycle_type = fields['CT_DR'].get(lad, 'Reserved / unknown')

        # TODO: Warning/error on invalid cycle types.
        if 'Reserved' in self.cycle_type:
            self.putb([0, ['Invalid cycle type (%s)' % lad_bits]])

        self.es_block = self.samplenum
        self.putb([2, ['Cycle type: %s' % self.cycle_type]])
        self.ss_block = self.samplenum

        self.state = 'GET ADDR'
        self.addr = 0
        self.cur_nibble = 0

    def handle_get_addr(self, lad, lad_bits):
        # LAD[3:0]: ADDR field (4/8/0 clock cycles).

        # I/O cycles: 4 ADDR clocks. Memory cycles: 8 ADDR clocks.
        # DMA cycles: no ADDR clocks at all.
        if self.cycle_type in ('I/O read', 'I/O write'):
            addr_nibbles = 4 # Address is 16bits.
        elif self.cycle_type in ('Memory read', 'Memory write'):
            addr_nibbles = 8 # Address is 32bits.
        else:
            addr_nibbles = 0 # TODO: How to handle later on?

        # Addresses are driven MSN-first.
        offset = ((addr_nibbles - 1) - self.cur_nibble) * 4
        self.addr |= (lad << offset)

        # Continue if we haven't seen all ADDR cycles, yet.
        if (self.cur_nibble < addr_nibbles - 1):
            self.cur_nibble += 1
            return

        self.es_block = self.samplenum
        s = 'Address: 0x%%0%dx' % addr_nibbles
        self.putb([3, [s % self.addr]])
        self.ss_block = self.samplenum

        self.state = 'GET TAR'
        self.tar_count = 0

    def handle_get_tar(self, lad, lad_bits):
        # LAD[3:0]: First TAR (turn-around) field (2 clock cycles).

        self.es_block = self.samplenum
        self.putb([4, ['TAR, cycle %d: %s' % (self.tarcount, lad_bits)]])
        self.ss_block = self.samplenum

        # On the first TAR clock cycle LAD[3:0] is driven to 1111 by
        # either the host or peripheral. On the second clock cycle,
        # the host or peripheral tri-states LAD[3:0], but its value
        # should still be 1111, due to pull-ups on the LAD lines.
        if lad_bits != '1111':
            self.putb([0, ['TAR, cycle %d: %s (expected 1111)' % \
                           (self.tarcount, lad_bits)]])

        if (self.tarcount != 1):
            self.tarcount += 1
            return

        self.tarcount = 0
        self.state = 'GET SYNC'

    def handle_get_sync(self, lad, lad_bits):
        # LAD[3:0]: SYNC field (1-n clock cycles).

        self.sync_val = lad_bits
        self.cycle_type = fields['SYNC'].get(lad, 'Reserved / unknown')

        # TODO: Warnings if reserved value are seen?
        if 'Reserved' in self.cycle_type:
            self.putb([0, ['SYNC, cycle %d: %s (reserved value)' % \
                           (self.synccount, self.sync_val)]])

        self.es_block = self.samplenum
        self.putb([5, ['SYNC, cycle %d: %s' % (self.synccount, self.sync_val)]])
        self.ss_block = self.samplenum

        # TODO

        self.cycle_count = 0
        self.state = 'GET DATA'

    def handle_get_data(self, lad, lad_bits):
        # LAD[3:0]: DATA field (2 clock cycles).

        # Data is driven LSN-first.
        if (self.cycle_count == 0):
            self.databyte = lad
        elif (self.cycle_count == 1):
            self.databyte |= (lad << 4)
        else:
            raise Exception('Invalid cycle_count: %d' % self.cycle_count)

        if (self.cycle_count != 1):
            self.cycle_count += 1
            return

        self.es_block = self.samplenum
        self.putb([6, ['DATA: 0x%02x' % self.databyte]])
        self.ss_block = self.samplenum

        self.cycle_count = 0
        self.state = 'GET TAR2'

    def handle_get_tar2(self, lad, lad_bits):
        # LAD[3:0]: Second TAR field (2 clock cycles).

        self.es_block = self.samplenum
        self.putb([7, ['TAR, cycle %d: %s' % (self.tarcount, lad_bits)]])
        self.ss_block = self.samplenum

        # On the first TAR clock cycle LAD[3:0] is driven to 1111 by
        # either the host or peripheral. On the second clock cycle,
        # the host or peripheral tri-states LAD[3:0], but its value
        # should still be 1111, due to pull-ups on the LAD lines.
        if lad_bits != '1111':
            self.putb([0, ['Warning: TAR, cycle %d: %s (expected 1111)'
                           % (self.tarcount, lad_bits)]])

        if (self.tarcount != 1):
            self.tarcount += 1
            return

        self.tarcount = 0
        self.state = 'IDLE'

    def decode(self):
        while True:
            # TODO: Come up with more appropriate self.wait() conditions.
            pins = self.wait()

            # If none of the pins changed, there's nothing to do.
            if self.oldpins == pins:
                continue

            # Store current pin values for the next round.
            self.oldpins = pins

            # Get individual pin values into local variables.
            (lframe, lclk, lad0, lad1, lad2, lad3) = pins[:6]
            (lreset, ldrq, serirq, clkrun, lpme, lpcpd, lsmi) = pins[6:]

            # Only look at the signals upon rising LCLK edges. The LPC clock
            # is the same as the PCI clock (which is sampled at rising edges).
            if not (self.oldlclk == 0 and lclk == 1):
                self.oldlclk = lclk
                continue

            # Store LAD[3:0] bit values (one nibble) in local variables.
            # Most (but not all) states need this.
            if self.state != 'IDLE':
                lad = (lad3 << 3) | (lad2 << 2) | (lad1 << 1) | lad0
                lad_bits = '{:04b}'.format(lad)
                # self.putb([0, ['LAD: %s' % lad_bits]])

            # TODO: Only memory read/write is currently supported/tested.

            # State machine
            if self.state == 'IDLE':
                # A valid LPC cycle starts with LFRAME# being asserted (low).
                if lframe != 0:
                    continue
                self.ss_block = self.samplenum
                self.state = 'GET START'
                self.lad = -1
            elif self.state == 'GET START':
                self.handle_get_start(lad, lad_bits, lframe)
            elif self.state == 'GET CT/DR':
                self.handle_get_ct_dr(lad, lad_bits)
            elif self.state == 'GET ADDR':
                self.handle_get_addr(lad, lad_bits)
            elif self.state == 'GET TAR':
                self.handle_get_tar(lad, lad_bits)
            elif self.state == 'GET SYNC':
                self.handle_get_sync(lad, lad_bits)
            elif self.state == 'GET DATA':
                self.handle_get_data(lad, lad_bits)
            elif self.state == 'GET TAR2':
                self.handle_get_tar2(lad, lad_bits)



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2014 Sebastien Bourdelin <sebastien.bourdelin@savoirfairelinux.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

# Helper dictionary for edge detection.
edge_detector = {
    'rising':  lambda x, y: bool(not x and y),
    'falling': lambda x, y: bool(x and not y),
    'both':    lambda x, y: bool(x ^ y),
}

class SamplerateError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'jitter'
    name = 'Jitter'
    longname = 'Timing jitter calculation'
    desc = 'Retrieves the timing jitter between two digital signals.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Clock/timing', 'Util']
    channels = (
        {'id': 'clk', 'name': 'Clock', 'desc': 'Clock reference channel'},
        {'id': 'sig', 'name': 'Resulting signal', 'desc': 'Resulting signal controlled by the clock'},
    )
    options = (
        {'id': 'clk_polarity', 'desc': 'Clock edge polarity',
            'default': 'rising', 'values': ('rising', 'falling', 'both')},
        {'id': 'sig_polarity', 'desc': 'Resulting signal edge polarity',
            'default': 'rising', 'values': ('rising', 'falling', 'both')},
    )
    annotations = (
        ('jitter', 'Jitter value'),
        ('clk_missed', 'Clock missed'),
        ('sig_missed', 'Signal missed'),
    )
    annotation_rows = (
        ('jitter', 'Jitter values', (0,)),
        ('clk_missed', 'Clock missed', (1,)),
        ('sig_missed', 'Signal missed', (2,)),
    )
    binary = (
        ('ascii-float', 'Jitter values as newline-separated ASCII floats'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.state = 'CLK'
        self.samplerate = None
        self.oldclk, self.oldsig = 0, 0
        self.clk_start = None
        self.sig_start = None
        self.clk_missed = 0
        self.sig_missed = 0

    def start(self):
        self.clk_edge = edge_detector[self.options['clk_polarity']]
        self.sig_edge = edge_detector[self.options['sig_polarity']]
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_binary = self.register(srd.OUTPUT_BINARY)
        self.out_clk_missed = self.register(srd.OUTPUT_META,
            meta=(int, 'Clock missed', 'Clock transition missed'))
        self.out_sig_missed = self.register(srd.OUTPUT_META,
            meta=(int, 'Signal missed', 'Resulting signal transition missed'))

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    # Helper function for jitter time annotations.
    def putx(self, delta):
        # Adjust granularity.
        if delta == 0 or delta >= 1:
            delta_s = '%.1fs' % (delta)
        elif delta <= 1e-12:
            delta_s = '%.1ffs' % (delta * 1e15)
        elif delta <= 1e-9:
            delta_s = '%.1fps' % (delta * 1e12)
        elif delta <= 1e-6:
            delta_s = '%.1fns' % (delta * 1e9)
        elif delta <= 1e-3:
            delta_s = '%.1fμs' % (delta * 1e6)
        else:
            delta_s = '%.1fms' % (delta * 1e3)

        self.put(self.clk_start, self.sig_start, self.out_ann, [0, [delta_s]])

    # Helper function for ASCII float jitter values (one value per line).
    def putb(self, delta):
        if delta is None:
            return
        # Format the delta to an ASCII float value terminated by a newline.
        x = str(delta) + '\n'
        self.put(self.clk_start, self.sig_start, self.out_binary,
                 [0, x.encode('UTF-8')])

    # Helper function for missed clock and signal annotations.
    def putm(self, data):
        self.put(self.samplenum, self.samplenum, self.out_ann, data)

    def handle_clk(self, clk, sig):
        if self.clk_start == self.samplenum:
            # Clock transition already treated.
            # We have done everything we can with this sample.
            return True

        if self.clk_edge(self.oldclk, clk):
            # Clock edge found.
            # We note the sample and move to the next state.
            self.clk_start = self.samplenum
            self.state = 'SIG'
            return False
        else:
            if self.sig_start is not None \
               and self.sig_start != self.samplenum \
               and self.sig_edge(self.oldsig, sig):
                # If any transition in the resulting signal
                # occurs while we are waiting for a clock,
                # we increase the missed signal counter.
                self.sig_missed += 1
                self.put(self.samplenum, self.samplenum, self.out_sig_missed, self.sig_missed)
                self.putm([2, ['Missed signal', 'MS']])
            # No clock edge found, we have done everything we
            # can with this sample.
            return True

    def handle_sig(self, clk, sig):
        if self.sig_start == self.samplenum:
            # Signal transition already treated.
            # We have done everything we can with this sample.
            return True

        if self.sig_edge(self.oldsig, sig):
            # Signal edge found.
            # We note the sample, calculate the jitter
            # and move to the next state.
            self.sig_start = self.samplenum
            self.state = 'CLK'
            # Calculate and report the timing jitter.
            delta = (self.sig_start - self.clk_start) / self.samplerate
            self.putx(delta)
            self.putb(delta)
            return False
        else:
            if self.clk_start != self.samplenum \
               and self.clk_edge(self.oldclk, clk):
                # If any transition in the clock signal
                # occurs while we are waiting for a resulting
                # signal, we increase the missed clock counter.
                self.clk_missed += 1
                self.put(self.samplenum, self.samplenum, self.out_clk_missed, self.clk_missed)
                self.putm([1, ['Missed clock', 'MC']])
            # No resulting signal edge found, we have done
            # everything we can with this sample.
            return True

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')
        while True:
            # Wait for a transition on CLK and/or SIG.
            clk, sig = self.wait([{0: 'e'}, {1: 'e'}])

            # State machine:
            # For each sample we can move 2 steps forward in the state machine.
            while True:
                # Clock state has the lead.
                if self.state == 'CLK':
                    if self.handle_clk(clk, sig):
                        break
                if self.state == 'SIG':
                    if self.handle_sig(clk, sig):
                        break

            # Save current CLK/SIG values for the next round.
            self.oldclk, self.oldsig = clk, sig



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Gerhard Sittig <gerhard.sittig@gmx.net>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
AC'97 (Audio Codec '97) is a protocol for audio and modem I/O functionality
in mainstream PC systems.

AC'97 communicates full duplex data (SDATA_IN, SDATA_OUT), where bits
are clocked by the BIT_CLK and frames are signalled by the SYNC signals.
A low active RESET# line completes the set of signals.

Frames repeat at a nominal frequency of 48kHz, and consist of 256 bits
each. One 16bit slot contains management information, twelve 20bit slots
follow which carry data for three management and nine audio/modem channels.
Optionally two slots of one frame can get combined for higher resolution
on fewer channels, or double data rate.

Details:
http://download.intel.com/support/motherboards/desktop/sb/ac97_r23.pdf
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2011-2014 Uwe Hermann <uwe@hermann-uwe.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from common.srdhelper import bitpack
from math import floor, ceil

'''
OUTPUT_PYTHON format:

Packet:
[<ptype>, <rxtx>, <pdata>]

This is the list of <ptype>s and their respective <pdata> values:
 - 'STARTBIT': The data is the (integer) value of the start bit (0/1).
 - 'DATA': This is always a tuple containing two items:
   - 1st item: the (integer) value of the UART data. Valid values
     range from 0 to 511 (as the data can be up to 9 bits in size).
   - 2nd item: the list of individual data bits and their ss/es numbers.
 - 'PARITYBIT': The data is the (integer) value of the parity bit (0/1).
 - 'STOPBIT': The data is the (integer) value of the stop bit (0 or 1).
 - 'INVALID STARTBIT': The data is the (integer) value of the start bit (0/1).
 - 'INVALID STOPBIT': The data is the (integer) value of the stop bit (0/1).
 - 'PARITY ERROR': The data is a tuple with two entries. The first one is
   the expected parity value, the second is the actual parity value.
 - 'BREAK': The data is always 0.
 - 'FRAME': The data is always a tuple containing two items: The (integer)
   value of the UART data, and a boolean which reflects the validity of the
   UART frame.
 - 'IDLE': The data is always 0.

The <rxtx> field is 0 for RX packets, 1 for TX packets.
'''

# Used for differentiating between the two data directions.
RX = 0
TX = 1

# Given a parity type to check (odd, even, zero, one), the value of the
# parity bit, the value of the data, and the length of the data (5-9 bits,
# usually 8 bits) return True if the parity is correct, False otherwise.
# 'none' is _not_ allowed as value for 'parity_type'.
def parity_ok(parity_type, parity_bit, data, data_bits):

    if parity_type == 'ignore':
        return True

    # Handle easy cases first (parity bit is always 1 or 0).
    if parity_type == 'zero':
        return parity_bit == 0
    elif parity_type == 'one':
        return parity_bit == 1

    # Count number of 1 (high) bits in the data (and the parity bit itself!).
    ones = bin(data).count('1') + parity_bit

    # Check for odd/even parity.
    if parity_type == 'odd':
        return (ones % 2) == 1
    elif parity_type == 'even':
        return (ones % 2) == 0

class SamplerateError(Exception):
    pass

class ChannelError(Exception):
    pass

class Decoder(srd.Decoder):
    api_version = 3
    id = 'uart'
    name = 'UART'
    longname = 'Universal Asynchronous Receiver/Transmitter'
    desc = 'Asynchronous, serial bus.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = ['uart']
    tags = ['Embedded/industrial']
    optional_channels = (
        # Allow specifying only one of the signals, e.g. if only one data
        # direction exists (or is relevant).
        {'id': 'rx', 'name': 'RX', 'desc': 'UART receive line'},
        {'id': 'tx', 'name': 'TX', 'desc': 'UART transmit line'},
    )
    options = (
        {'id': 'baudrate', 'desc': 'Baud rate', 'default': 115200},
        {'id': 'data_bits', 'desc': 'Data bits', 'default': 8,
            'values': (5, 6, 7, 8, 9)},
        {'id': 'parity', 'desc': 'Parity', 'default': 'none',
            'values': ('none', 'odd', 'even', 'zero', 'one', 'ignore')},
        {'id': 'stop_bits', 'desc': 'Stop bits', 'default': 1.0,
            'values': (0.0, 0.5, 1.0, 1.5)},
        {'id': 'bit_order', 'desc': 'Bit order', 'default': 'lsb-first',
            'values': ('lsb-first', 'msb-first')},
        {'id': 'format', 'desc': 'Data format', 'default': 'hex',
            'values': ('ascii', 'dec', 'hex', 'oct', 'bin')},
        {'id': 'invert_rx', 'desc': 'Invert RX', 'default': 'no',
            'values': ('yes', 'no')},
        {'id': 'invert_tx', 'desc': 'Invert TX', 'default': 'no',
            'values': ('yes', 'no')},
        {'id': 'rx_packet_delim', 'desc': 'RX packet delimiter (decimal)',
            'default': -1},
        {'id': 'tx_packet_delim', 'desc': 'TX packet delimiter (decimal)',
            'default': -1},
        {'id': 'rx_packet_len', 'desc': 'RX packet length', 'default': -1},
        {'id': 'tx_packet_len', 'desc': 'TX packet length', 'default': -1},
    )
    annotations = (
        ('rx-data', 'RX data'),
        ('tx-data', 'TX data'),
        ('rx-start', 'RX start bits'),
        ('tx-start', 'TX start bits'),
        ('rx-parity-ok', 'RX parity OK bits'),
        ('tx-parity-ok', 'TX parity OK bits'),
        ('rx-parity-err', 'RX parity error bits'),
        ('tx-parity-err', 'TX parity error bits'),
        ('rx-stop', 'RX stop bits'),
        ('tx-stop', 'TX stop bits'),
        ('rx-warnings', 'RX warnings'),
        ('tx-warnings', 'TX warnings'),
        ('rx-data-bits', 'RX data bits'),
        ('tx-data-bits', 'TX data bits'),
        ('rx-break', 'RX break'),
        ('tx-break', 'TX break'),
        ('rx-packet', 'RX packet'),
        ('tx-packet', 'TX packet'),
    )
    annotation_rows = (
        ('rx-data-bits', 'RX bits', (12,)),
        ('rx-data', 'RX', (0, 2, 4, 6, 8)),
        ('rx-warnings', 'RX warnings', (10,)),
        ('rx-break', 'RX break', (14,)),
        ('rx-packets', 'RX packets', (16,)),
        ('tx-data-bits', 'TX bits', (13,)),
        ('tx-data', 'TX', (1, 3, 5, 7, 9)),
        ('tx-warnings', 'TX warnings', (11,)),
        ('tx-break', 'TX break', (15,)),
        ('tx-packets', 'TX packets', (17,)),
    )
    binary = (
        ('rx', 'RX dump'),
        ('tx', 'TX dump'),
        ('rxtx', 'RX/TX dump'),
    )
    idle_state = ['WAIT FOR START BIT', 'WAIT FOR START BIT']

    def putx(self, rxtx, data):
        s, halfbit = self.startsample[rxtx], self.bit_width / 2.0
        self.put(s - floor(halfbit), self.samplenum + ceil(halfbit), self.out_ann, data)

    def putx_packet(self, rxtx, data):
        s, halfbit = self.ss_packet[rxtx], self.bit_width / 2.0
        self.put(s - floor(halfbit), self.samplenum + ceil(halfbit), self.out_ann, data)

    def putpx(self, rxtx, data):
        s, halfbit = self.startsample[rxtx], self.bit_width / 2.0
        self.put(s - floor(halfbit), self.samplenum + ceil(halfbit), self.out_python, data)

    def putg(self, data):
        s, halfbit = self.samplenum, self.bit_width / 2.0
        self.put(s - floor(halfbit), s + ceil(halfbit), self.out_ann, data)

    def putp(self, data):
        s, halfbit = self.samplenum, self.bit_width / 2.0
        self.put(s - floor(halfbit), s + ceil(halfbit), self.out_python, data)

    def putgse(self, ss, es, data):
        self.put(ss, es, self.out_ann, data)

    def putpse(self, ss, es, data):
        self.put(ss, es, self.out_python, data)

    def putbin(self, rxtx, data):
        s, halfbit = self.startsample[rxtx], self.bit_width / 2.0
        self.put(s - floor(halfbit), self.samplenum + ceil(halfbit), self.out_binary, data)

    def __init__(self):
        self.reset()

    def reset(self):
        self.samplerate = None
        self.frame_start = [-1, -1]
        self.frame_valid = [None, None]
        self.startbit = [-1, -1]
        self.cur_data_bit = [0, 0]
        self.datavalue = [0, 0]
        self.paritybit = [-1, -1]
        self.stopbit1 = [-1, -1]
        self.startsample = [-1, -1]
        self.state = ['WAIT FOR START BIT', 'WAIT FOR START BIT']
        self.databits = [[], []]
        self.break_start = [None, None]
        self.packet_cache = [[], []]
        self.ss_packet, self.es_packet = [None, None], [None, None]
        self.idle_start = [None, None]

    def start(self):
        self.out_python = self.register(srd.OUTPUT_PYTHON)
        self.out_binary = self.register(srd.OUTPUT_BINARY)
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.bw = (self.options['data_bits'] + 7) // 8

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value
            # The width of one UART bit in number of samples.
            self.bit_width = float(self.samplerate) / float(self.options['baudrate'])

    def get_sample_point(self, rxtx, bitnum):
        # Determine absolute sample number of a bit slot's sample point.
        # bitpos is the samplenumber which is in the middle of the
        # specified UART bit (0 = start bit, 1..x = data, x+1 = parity bit
        # (if used) or the first stop bit, and so on).
        # The samples within bit are 0, 1, ..., (bit_width - 1), therefore
        # index of the middle sample within bit window is (bit_width - 1) / 2.
        bitpos = self.frame_start[rxtx] + (self.bit_width - 1) / 2.0
        bitpos += bitnum * self.bit_width
        return bitpos

    def wait_for_start_bit(self, rxtx, signal):
        # Save the sample number where the start bit begins.
        self.frame_start[rxtx] = self.samplenum
        self.frame_valid[rxtx] = True

        self.state[rxtx] = 'GET START BIT'

    def get_start_bit(self, rxtx, signal):
        self.startbit[rxtx] = signal

        # The startbit must be 0. If not, we report an error and wait
        # for the next start bit (assuming this one was spurious).
        if self.startbit[rxtx] != 0:
            self.putp(['INVALID STARTBIT', rxtx, self.startbit[rxtx]])
            self.putg([rxtx + 10, ['Frame error', 'Frame err', 'FE']])
            self.frame_valid[rxtx] = False
            es = self.samplenum + ceil(self.bit_width / 2.0)
            self.putpse(self.frame_start[rxtx], es, ['FRAME', rxtx,
                (self.datavalue[rxtx], self.frame_valid[rxtx])])
            self.state[rxtx] = 'WAIT FOR START BIT'
            return

        self.cur_data_bit[rxtx] = 0
        self.datavalue[rxtx] = 0
        self.startsample[rxtx] = -1

        self.putp(['STARTBIT', rxtx, self.startbit[rxtx]])
        self.putg([rxtx + 2, ['Start bit', 'Start', 'S']])

        self.state[rxtx] = 'GET DATA BITS'

    def handle_packet(self, rxtx):
        d = 'rx' if (rxtx == RX) else 'tx'
        delim = self.options[d + '_packet_delim']
        plen = self.options[d + '_packet_len']
        if delim == -1 and plen == -1:
            return

        # Cache data values until we see the delimiter and/or the specified
        # packet length has been reached (whichever happens first).
        if len(self.packet_cache[rxtx]) == 0:
            self.ss_packet[rxtx] = self.startsample[rxtx]
        self.packet_cache[rxtx].append(self.datavalue[rxtx])
        if self.datavalue[rxtx] == delim or len(self.packet_cache[rxtx]) == plen:
            self.es_packet[rxtx] = self.samplenum
            s = ''
            for b in self.packet_cache[rxtx]:
                s += self.format_value(b)
                if self.options['format'] != 'ascii':
                    s += ' '
            if self.options['format'] != 'ascii' and s[-1] == ' ':
                s = s[:-1] # Drop trailing space.
            self.putx_packet(rxtx, [16 + rxtx, [s]])
            self.packet_cache[rxtx] = []

    def get_data_bits(self, rxtx, signal):
        # Save the sample number of the middle of the first data bit.
        if self.startsample[rxtx] == -1:
            self.startsample[rxtx] = self.samplenum

        self.putg([rxtx + 12, ['%d' % signal]])

        # Store individual data bits and their start/end samplenumbers.
        s, halfbit = self.samplenum, int(self.bit_width / 2)
        self.databits[rxtx].append([signal, s - halfbit, s + halfbit])

        # Return here, unless we already received all data bits.
        self.cur_data_bit[rxtx] += 1
        if self.cur_data_bit[rxtx] < self.options['data_bits']:
            return

        # Convert accumulated data bits to a data value.
        bits = [b[0] for b in self.databits[rxtx]]
        if self.options['bit_order'] == 'msb-first':
            bits.reverse()
        self.datavalue[rxtx] = bitpack(bits)
        self.putpx(rxtx, ['DATA', rxtx,
            (self.datavalue[rxtx], self.databits[rxtx])])

        b = self.datavalue[rxtx]
        formatted = self.format_value(b)
        if formatted is not None:
            self.putx(rxtx, [rxtx, [formatted]])

        bdata = b.to_bytes(self.bw, byteorder='big')
        self.putbin(rxtx, [rxtx, bdata])
        self.putbin(rxtx, [2, bdata])

        self.handle_packet(rxtx)

        self.databits[rxtx] = []

        # Advance to either reception of the parity bit, or reception of
        # the STOP bits if parity is not applicable.
        self.state[rxtx] = 'GET PARITY BIT'
        if self.options['parity'] == 'none':
            self.state[rxtx] = 'GET STOP BITS'

    def format_value(self, v):
        # Format value 'v' according to configured options.
        # Reflects the user selected kind of representation, as well as
        # the number of data bits in the UART frames.

        fmt, bits = self.options['format'], self.options['data_bits']

        # Assume "is printable" for values from 32 to including 126,
        # below 32 is "control" and thus not printable, above 127 is
        # "not ASCII" in its strict sense, 127 (DEL) is not printable,
        # fall back to hex representation for non-printables.
        if fmt == 'ascii':
            if v in range(32, 126 + 1):
                return chr(v)
            hexfmt = "[{:02X}]" if bits <= 8 else "[{:03X}]"
            return hexfmt.format(v)

        # Mere number to text conversion without prefix and padding
        # for the "decimal" output format.
        if fmt == 'dec':
            return "{:d}".format(v)

        # Padding with leading zeroes for hex/oct/bin formats, but
        # without a prefix for density -- since the format is user
        # specified, there is no ambiguity.
        if fmt == 'hex':
            digits = (bits + 4 - 1) // 4
            fmtchar = "X"
        elif fmt == 'oct':
            digits = (bits + 3 - 1) // 3
            fmtchar = "o"
        elif fmt == 'bin':
            digits = bits
            fmtchar = "b"
        else:
            fmtchar = None
        if fmtchar is not None:
            fmt = "{{:0{:d}{:s}}}".format(digits, fmtchar)
            return fmt.format(v)

        return None

    def get_parity_bit(self, rxtx, signal):
        self.paritybit[rxtx] = signal

        if parity_ok(self.options['parity'], self.paritybit[rxtx],
                     self.datavalue[rxtx], self.options['data_bits']):
            self.putp(['PARITYBIT', rxtx, self.paritybit[rxtx]])
            self.putg([rxtx + 4, ['Parity bit', 'Parity', 'P']])
        else:
            # TODO: Return expected/actual parity values.
            self.putp(['PARITY ERROR', rxtx, (0, 1)]) # FIXME: Dummy tuple...
            self.putg([rxtx + 6, ['Parity error', 'Parity err', 'PE']])
            self.frame_valid[rxtx] = False

        self.state[rxtx] = 'GET STOP BITS'

    # TODO: Currently only supports 1 stop bit.
    def get_stop_bits(self, rxtx, signal):
        self.stopbit1[rxtx] = signal

        # Stop bits must be 1. If not, we report an error.
        if self.stopbit1[rxtx] != 1:
            self.putp(['INVALID STOPBIT', rxtx, self.stopbit1[rxtx]])
            self.putg([rxtx + 10, ['Frame error', 'Frame err', 'FE']])
            self.frame_valid[rxtx] = False

        self.putp(['STOPBIT', rxtx, self.stopbit1[rxtx]])
        self.putg([rxtx + 4, ['Stop bit', 'Stop', 'T']])

        # Pass the complete UART frame to upper layers.
        es = self.samplenum + ceil(self.bit_width / 2.0)
        self.putpse(self.frame_start[rxtx], es, ['FRAME', rxtx,
            (self.datavalue[rxtx], self.frame_valid[rxtx])])

        self.state[rxtx] = 'WAIT FOR START BIT'
        self.idle_start[rxtx] = self.frame_start[rxtx] + self.frame_len_sample_count

    def handle_break(self, rxtx):
        self.putpse(self.frame_start[rxtx], self.samplenum,
                ['BREAK', rxtx, 0])
        self.putgse(self.frame_start[rxtx], self.samplenum,
                [rxtx + 14, ['Break condition', 'Break', 'Brk', 'B']])
        self.state[rxtx] = 'WAIT FOR START BIT'

    def get_wait_cond(self, rxtx, inv):
        # Return condititions that are suitable for Decoder.wait(). Those
        # conditions either match the falling edge of the START bit, or
        # the sample point of the next bit time.
        state = self.state[rxtx]
        if state == 'WAIT FOR START BIT':
            return {rxtx: 'r' if inv else 'f'}
        if state == 'GET START BIT':
            bitnum = 0
        elif state == 'GET DATA BITS':
            bitnum = 1 + self.cur_data_bit[rxtx]
        elif state == 'GET PARITY BIT':
            bitnum = 1 + self.options['data_bits']
        elif state == 'GET STOP BITS':
            bitnum = 1 + self.options['data_bits']
            bitnum += 0 if self.options['parity'] == 'none' else 1
        want_num = ceil(self.get_sample_point(rxtx, bitnum))
        return {'skip': want_num - self.samplenum}

    def get_idle_cond(self, rxtx, inv):
        # Return a condition that corresponds to the (expected) end of
        # the next frame, assuming that it will be an "idle frame"
        # (constant high input level for the frame's length).
        if self.idle_start[rxtx] is None:
            return None
        end_of_frame = self.idle_start[rxtx] + self.frame_len_sample_count
        if end_of_frame < self.samplenum:
            return None
        return {'skip': end_of_frame - self.samplenum}

    def inspect_sample(self, rxtx, signal, inv):
        # Inspect a sample returned by .wait() for the specified UART line.
        if inv:
            signal = not signal

        state = self.state[rxtx]
        if state == 'WAIT FOR START BIT':
            self.wait_for_start_bit(rxtx, signal)
        elif state == 'GET START BIT':
            self.get_start_bit(rxtx, signal)
        elif state == 'GET DATA BITS':
            self.get_data_bits(rxtx, signal)
        elif state == 'GET PARITY BIT':
            self.get_parity_bit(rxtx, signal)
        elif state == 'GET STOP BITS':
            self.get_stop_bits(rxtx, signal)

    def inspect_edge(self, rxtx, signal, inv):
        # Inspect edges, independently from traffic, to detect break conditions.
        if inv:
            signal = not signal
        if not signal:
            # Signal went low. Start another interval.
            self.break_start[rxtx] = self.samplenum
            return
        # Signal went high. Was there an extended period with low signal?
        if self.break_start[rxtx] is None:
            return
        diff = self.samplenum - self.break_start[rxtx]
        if diff >= self.break_min_sample_count:
            self.handle_break(rxtx)
        self.break_start[rxtx] = None

    def inspect_idle(self, rxtx, signal, inv):
        # Check each edge and each period of stable input (either level).
        # Can derive the "idle frame period has passed" condition.
        if inv:
            signal = not signal
        if not signal:
            # Low input, cease inspection.
            self.idle_start[rxtx] = None
            return
        # High input, either just reached, or still stable.
        if self.idle_start[rxtx] is None:
            self.idle_start[rxtx] = self.samplenum
        diff = self.samplenum - self.idle_start[rxtx]
        if diff < self.frame_len_sample_count:
            return
        ss, es = self.idle_start[rxtx], self.samplenum
        self.putpse(ss, es, ['IDLE', rxtx, 0])
        self.idle_start[rxtx] = self.samplenum

    def decode(self):
        if not self.samplerate:
            raise SamplerateError('Cannot decode without samplerate.')

        has_pin = [self.has_channel(ch) for ch in (RX, TX)]
        if not True in has_pin:
            raise ChannelError('Need at least one of TX or RX pins.')

        opt = self.options
        inv = [opt['invert_rx'] == 'yes', opt['invert_tx'] == 'yes']
        cond_data_idx = [None] * len(has_pin)

        # Determine the number of samples for a complete frame's time span.
        # A period of low signal (at least) that long is a break condition.
        frame_samples = 1 # START
        frame_samples += self.options['data_bits']
        frame_samples += 0 if self.options['parity'] == 'none' else 1
        frame_samples += self.options['stop_bits']
        frame_samples *= self.bit_width
        self.frame_len_sample_count = ceil(frame_samples)
        self.break_min_sample_count = self.frame_len_sample_count
        cond_edge_idx = [None] * len(has_pin)
        cond_idle_idx = [None] * len(has_pin)

        while True:
            conds = []
            if has_pin[RX]:
                cond_data_idx[RX] = len(conds)
                conds.append(self.get_wait_cond(RX, inv[RX]))
                cond_edge_idx[RX] = len(conds)
                conds.append({RX: 'e'})
                cond_idle_idx[RX] = None
                idle_cond = self.get_idle_cond(RX, inv[RX])
                if idle_cond:
                    cond_idle_idx[RX] = len(conds)
                    conds.append(idle_cond)
            if has_pin[TX]:
                cond_data_idx[TX] = len(conds)
                conds.append(self.get_wait_cond(TX, inv[TX]))
                cond_edge_idx[TX] = len(conds)
                conds.append({TX: 'e'})
                cond_idle_idx[TX] = None
                idle_cond = self.get_idle_cond(TX, inv[TX])
                if idle_cond:
                    cond_idle_idx[TX] = len(conds)
                    conds.append(idle_cond)
            (rx, tx) = self.wait(conds)
            if cond_data_idx[RX] is not None and self.matched[cond_data_idx[RX]]:
                self.inspect_sample(RX, rx, inv[RX])
            if cond_edge_idx[RX] is not None and self.matched[cond_edge_idx[RX]]:
                self.inspect_edge(RX, rx, inv[RX])
                self.inspect_idle(RX, rx, inv[RX])
            if cond_idle_idx[RX] is not None and self.matched[cond_idle_idx[RX]]:
                self.inspect_idle(RX, rx, inv[RX])
            if cond_data_idx[TX] is not None and self.matched[cond_data_idx[TX]]:
                self.inspect_sample(TX, tx, inv[TX])
            if cond_edge_idx[TX] is not None and self.matched[cond_edge_idx[TX]]:
                self.inspect_edge(TX, tx, inv[TX])
                self.inspect_idle(TX, tx, inv[TX])
            if cond_idle_idx[TX] is not None and self.matched[cond_idle_idx[TX]]:
                self.inspect_idle(TX, tx, inv[TX])



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Kevin Redon <kingkevin@cuvoodoo.info>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

'''
Microwire is a 3-wire half-duplex synchronous serial communication protocol.

Originally from National Semiconductor, it is often used in EEPROM chips with
device specific commands on top of the bit stream.

Channels:

 - CS: chip select, active high
 - SK: clock line, for the synchronous communication (idle low)
 - SI: slave data input, output by the master and parsed by the selected slave
       on rising edge of clock line (idle low)
 - SO: slave data output, output by the selected slave and changed on rising
       edge of clock line, or goes from low to high when ready during status
       check (idle high impedance)

The channel names might vary from chip to chip but the underlying function is
the same.
'''

from .pd import Decoder



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2017 Marcus Comstedt <marcus@mc.pp.se>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

ann = [
    ['Size', 'L'],
    ['SrcAP', 'S'],
    ['DstAP', 'D'],
    ['Cmd', 'C'],
    ['Data'],
    ['Cksum', 'K'],
]

class Decoder(srd.Decoder):
    api_version = 3
    id = 'maple_bus'
    name = 'Maple bus'
    longname = 'SEGA Maple bus'
    desc = 'Maple bus peripheral protocol for SEGA Dreamcast.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Retro computing']
    channels = (
        {'id': 'sdcka', 'name': 'SDCKA', 'desc': 'Data/clock line A'},
        {'id': 'sdckb', 'name': 'SDCKB', 'desc': 'Data/clock line B'},
    )
    annotations = (
        ('start', 'Start pattern'),
        ('end', 'End pattern'),
        ('start-with-crc', 'Start pattern with CRC'),
        ('occupancy', 'SDCKB occupancy pattern'),
        ('reset', 'RESET pattern'),
        ('bit', 'Bit'),
        ('size', 'Data size'),
        ('source', 'Source AP'),
        ('dest', 'Destination AP'),
        ('command', 'Command'),
        ('data', 'Data'),
        ('checksum', 'Checksum'),
        ('frame-error', 'Frame error'),
        ('checksum-error', 'Checksum error'),
        ('size-error', 'Size error'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0, 1, 2, 3, 4, 5)),
        ('fields', 'Fields', (6, 7, 8, 9, 10, 11)),
        ('warnings', 'Warnings', (12, 13, 14)),
    )
    binary = (
        ('size', 'Data size'),
        ('source', 'Source AP'),
        ('dest', 'Destination AP'),
        ('command', 'Command code'),
        ('data', 'Data'),
        ('checksum', 'Checksum'),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        pass

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_binary = self.register(srd.OUTPUT_BINARY)
        self.pending_bit_pos = None

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    def putb(self, data):
        self.put(self.ss, self.es, self.out_binary, data)

    def byte_annotation(self, bintype, d):
        return [bintype + 6,
            ['%s: %02X' % (name, d) for name in ann[bintype]] + ['%02X' % d]]

    def got_start(self):
        self.putx([0, ['Start pattern', 'Start', 'S']])

    def got_end(self):
        self.putx([1, ['End pattern', 'End', 'E']])
        if self.length != self.expected_length + 1:
            self.putx([14, ['Size error', 'L error', 'LE']])

    def got_start_with_crc(self):
        self.putx([2, ['Start pattern with CRC', 'Start CRC', 'SC']])

    def got_occupancy(self):
        self.putx([3, ['SDCKB occupancy pattern', 'Occupancy', 'O']])

    def got_reset(self):
        self.putx([4, ['RESET pattern', 'RESET', 'R']])

    def output_pending_bit(self):
        if self.pending_bit_pos:
            self.put(self.pending_bit_pos, self.pending_bit_pos, self.out_ann, [5, ['Bit: %d' % self.pending_bit, '%d' % self.pending_bit]])

    def got_bit(self, n):
        self.output_pending_bit()
        self.data = self.data * 2 + n
        self.pending_bit = n
        self.pending_bit_pos = self.samplenum

    def got_byte(self):
        self.output_pending_bit()
        bintype = 4
        if self.length < 4:
            if self.length == 0:
                self.expected_length = 4 * (self.data + 1)
            bintype = self.length
        elif self.length == self.expected_length:
            bintype = 5
            if self.data != self.checksum:
                self.putx([13, ['Cksum error', 'K error', 'KE']])
        self.length = self.length + 1
        self.checksum = self.checksum ^ self.data
        self.putx(self.byte_annotation(bintype, self.data))
        self.putb([bintype, bytes([self.data])])
        self.pending_bit_pos = None

    def frame_error(self):
        self.putx([7, ['Frame error', 'F error', 'FE']])

    def handle_start(self):
        self.wait({0: 'l', 1: 'h'})
        self.ss = self.samplenum
        count = 0
        while True:
            sdcka, sdckb = self.wait([{1: 'f'}, {0: 'r'}])
            if self.matched[0]:
                count = count + 1
            if self.matched[1]:
                self.es = self.samplenum
                if sdckb == 1:
                    if count == 4:
                        self.got_start()
                        return True
                    elif count == 6:
                        self.got_start_with_crc()
                        return True
                    elif count == 8:
                        self.got_occupancy()
                        return False
                    elif count >= 14:
                        self.got_reset()
                        return False
                self.frame_error()
                return False

    def handle_byte_or_stop(self):
        self.ss = self.samplenum
        self.pending_bit_pos = None
        initial = True
        counta = 0
        countb = 0
        self.data = 0
        while countb < 4:
            sdcka, sdckb = self.wait([{0: 'f'}, {1: 'f'}])
            self.es = self.samplenum
            if self.matched[0]:
                if counta == countb:
                    self.got_bit(sdckb)
                    counta = counta + 1
                elif counta == 1 and countb == 0 and self.data == 0 and sdckb == 0:
                    self.wait([{0: 'h', 1: 'h'}, {0: 'f'}, {1: 'f'}])
                    self.es = self.samplenum
                    if self.matched[0]:
                        self.got_end()
                    else:
                        self.frame_error()
                    return False
                else:
                    self.frame_error()
                    return False
            elif self.matched[1]:
                if counta == countb + 1:
                    self.got_bit(sdcka)
                    countb = countb + 1
                elif counta == 0 and countb == 0 and sdcka == 1 and initial:
                    self.ss = self.samplenum
                    initial = False
                else:
                    self.frame_error()
                    return False
        self.wait({0: 'h'})
        self.es = self.samplenum
        self.got_byte()
        return True

    def decode(self):
        while True:
            while not self.handle_start():
                pass
            self.length = 0
            self.expected_length = 4
            self.checksum = 0
            while self.handle_byte_or_stop():
                pass



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Steve R <steversig@virginmedia.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd
from common.srdhelper import bcd2int

class Decoder(srd.Decoder):
    api_version = 3
    id = 'ook_vis'
    name = 'OOK visualisation'
    longname = 'On-off keying visualisation'
    desc = 'OOK visualisation in various formats.'
    license = 'gplv2+'
    inputs = ['ook']
    outputs = ['ook']
    tags = ['Encoding']
    annotations = (
        ('bit', 'Bit'),
        ('ref', 'Reference'),
        ('field', 'Field'),
        ('ref_field', 'Ref field'),
        ('level2', 'L2'),
        ('ref_level2', 'Ref L2'),
    )
    annotation_rows = (
        ('bits', 'Bits', (0,)),
        ('compare', 'Compare', (1,)),
        ('fields', 'Fields', (2,)),
        ('ref_fields', 'Ref fields', (3,)),
        ('level2', 'L2', (4,)),
        ('ref_level2', 'Ref L2', (5,)),
    )
    options = (
        {'id': 'displayas', 'desc': 'Display as', 'default': 'Nibble - Hex',
         'values': ('Byte - Hex', 'Byte - Hex rev', 'Byte - BCD',
         'Byte - BCD rev', 'Nibble - Hex', 'Nibble - Hex rev', 'Nibble - BCD',
         'Nibble - BCD rev')},
        {'id': 'synclen', 'desc': 'Sync length', 'default': '4',
         'values': ('0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '10')},
        {'id': 'syncoffset', 'desc': 'Sync offset', 'default': '0',
         'values': ('-4', '-3', '-2', '-1', '0', '1', '2', '3', '4')},
        {'id': 'refsample', 'desc': 'Compare', 'default': 'off', 'values':
        ('off', 'show numbers', '1', '2', '3', '4', '5', '6', '7', '8', '9',
         '10', '11', '12', '13', '14', '15', '16', '17', '18', '19', '20',
         '21', '22', '23', '24', '25', '26', '27', '28', '29', '30')},
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.decoded = [] # Local cache of decoded OOK.
        self.ookstring = ''
        self.ookcache = []
        self.trace_num = 0

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)
        self.out_python = self.register(srd.OUTPUT_PYTHON)
        self.displayas = self.options['displayas']
        self.sync_length = self.options['synclen']
        self.sync_offset = self.options['syncoffset']
        self.ref = self.options['refsample']

    def putx(self, data):
        self.put(self.ss, self.es, self.out_ann, data)

    def putp(self, data):
        self.put(self.ss, self.es, self.out_python, data)

    def display_level2(self, bits, line):
        self.decode_pos = 0
        ook = self.decoded
        # Find the end of the preamble which could be 1010 or 1111.
        if len(ook) > 1:
            preamble_end = len(ook) + 1
            char_first = ook[0][2]
            char_second = ook[1][2]
            if char_first == char_second: # 1111
                preamble = '1111'
                char_last = char_first
            else:
                preamble = '1010'
                char_last = char_second
            for i in range(len(ook)):
                if preamble == '1111':
                    if ook[i][2] != char_last:
                        preamble_end = i
                        break
                    else:
                        char_last = ook[i][2]
                else:
                    if ook[i][2] != char_last:
                        char_last = ook[i][2]
                    else:
                        preamble_end = i
                        break

            if len(ook) >= preamble_end:
                preamble_end += int(self.sync_offset) - 1
                self.ss, self.es = ook[0][0], ook[preamble_end][1]
                self.putx([line, ['Preamble', 'Pre', 'P']])
                self.decode_pos += preamble_end

                if len(ook) > self.decode_pos + int(self.sync_length):
                    self.ss = self.es
                    self.es = ook[self.decode_pos + int(self.sync_length)][1]
                    self.putx([line, ['Sync', 'Syn', 'S']])
                    self.decode_pos += int(self.sync_length) + 1

                ookstring = self.ookstring[self.decode_pos:]
                rem_nibbles = len(ookstring) // bits
                for i in range(rem_nibbles): # Display the rest of nibbles.
                    self.ss = ook[self.decode_pos][0]
                    self.es = ook[self.decode_pos + bits - 1][1]
                    self.put_field(bits, line)

    def put_field(self, numbits, line):
        param = self.ookstring[self.decode_pos:self.decode_pos + numbits]
        if 'rev' in self.displayas:
            param = param[::-1]     # Reversed from right.
        if not 'E' in param:        # Format if no errors.
            if 'Hex' in self.displayas:
                param = hex(int(param, 2))[2:]
            elif 'BCD' in self.displayas:
                param = bcd2int(int(param, 2))
        self.putx([line, [str(param)]])
        self.decode_pos += numbits

    def display_all(self):
        ookstring = ''
        self.decode_pos = 0
        ook = self.decoded
        for i in range(len(ook)):
            self.ookstring += ook[i][2]
        bits = 4 if 'Nibble' in self.displayas else 8
        rem_nibbles = len(self.ookstring) // bits
        for i in range(rem_nibbles): # Display the rest of the nibbles.
            self.ss = ook[self.decode_pos][0]
            self.es = ook[self.decode_pos + bits - 1][1]
            self.put_field(bits, 2)

        self.display_level2(bits, 4) # Display L2 decode.

        if (self.ref != 'off' and self.ref != 'show numbers' and
            len(self.ookcache) >= int(self.ref)): # Compare traces.
            ref = int(self.ref) - 1
            self.display_ref(self.trace_num, ref)
            if len(self.ookcache) == int(self.ref): # Backfill.
                for i in range(0, ref):
                    self.display_ref(i, ref)
        elif self.ref == 'show numbers': # Display ref numbers.
            self.ss = self.ookcache[self.trace_num][0][0]
            end_sig = len(self.ookcache[self.trace_num]) - 1
            self.es = self.ookcache[self.trace_num][end_sig][1]
            self.putx([1, [str(self.trace_num + 1)]])

    def display_ref(self, t_num, ref):
        display_len = len(self.ookcache[ref])
        if len(self.ookcache[t_num]) < len(self.ookcache[ref]):
            display_len = len(self.ookcache[t_num])
        for i in range(display_len):
            self.ss = self.ookcache[t_num][i][0]
            self.es = self.ookcache[t_num][i][1]
            self.putx([1, [self.ookcache[ref][i][2]]])

    def add_to_cache(self): # Cache the OOK so it can be used as a reference.
        self.ookcache.append(self.decoded)

    def decode(self, ss, es, data):
        self.decoded = data
        self.add_to_cache()
        self.display_all()
        self.ookstring = ''
        self.trace_num += 1
        self.ss = ss
        self.es = es
        self.putp(data) # Send data up the stack.



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2019 Benedikt Otto <benedikt_o@web.de>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import sigrokdecode as srd

class ChannelError(Exception):
    pass

digits = {
    (0, 0, 0, 0, 0, 0, 0): ' ',
    (1, 1, 1, 1, 1, 1, 0): '0',
    (0, 1, 1, 0, 0, 0, 0): '1',
    (1, 1, 0, 1, 1, 0, 1): '2',
    (1, 1, 1, 1, 0, 0, 1): '3',
    (0, 1, 1, 0, 0, 1, 1): '4',
    (1, 0, 1, 1, 0, 1, 1): '5',
    (1, 0, 1, 1, 1, 1, 1): '6',
    (1, 1, 1, 0, 0, 0, 0): '7',
    (1, 1, 1, 1, 1, 1, 1): '8',
    (1, 1, 1, 1, 0, 1, 1): '9',
    (1, 1, 1, 0, 1, 1, 1): 'A',
    (0, 0, 1, 1, 1, 1, 1): 'B',
    (1, 0, 0, 1, 1, 1, 0): 'C',
    (0, 1, 1, 1, 1, 0, 1): 'D',
    (1, 0, 0, 1, 1, 1, 1): 'E',
    (1, 0, 0, 0, 1, 1, 1): 'F',
}

class Decoder(srd.Decoder):
    api_version = 3
    id = 'seven_segment'
    name = '7-segment'
    longname = '7-segment display'
    desc = '7-segment display protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Display']
    channels = (
        {'id': 'a', 'name': 'A', 'desc': 'Segment A'},
        {'id': 'b', 'name': 'B', 'desc': 'Segment B'},
        {'id': 'c', 'name': 'C', 'desc': 'Segment C'},
        {'id': 'd', 'name': 'D', 'desc': 'Segment D'},
        {'id': 'e', 'name': 'E', 'desc': 'Segment E'},
        {'id': 'f', 'name': 'F', 'desc': 'Segment F'},
        {'id': 'g', 'name': 'G', 'desc': 'Segment G'},
    )
    optional_channels = (
        {'id': 'dp', 'name': 'DP', 'desc': 'Decimal point'},
    )
    options = (
        {'id': 'polarity', 'desc': 'Expected polarity',
            'default': 'common-cathode', 'values': ('common-cathode', 'common-anode')},
    )
    annotations = (
        ('decoded-digit', 'Decoded digit'),
    )
    annotation_rows = (
        ('decoded-digits', 'Decoded digits', (0,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        pass

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putb(self, ss_block, es_block, data):
        self.put(ss_block, es_block, self.out_ann, data)

    def pins_to_hex(self, pins):
        return digits.get(pins, None)

    def decode(self):
        oldpins = self.wait()

        # Check if at least the 7 signals are present.
        if False in [p in (0, 1) for p in oldpins[:7]]:
            raise ChannelError('7 or 8 pins have to be present.')

        lastpos = self.samplenum

        self.have_dp = self.has_channel(7)

        conditions = [{0: 'e'}, {1: 'e'}, {2: 'e'}, {3: 'e'}, {4: 'e'}, {5: 'e'}, {6: 'e'}]

        if self.have_dp:
            conditions.append({7: 'e'})

        while True:
            # Wait for any change.
            pins = self.wait(conditions)

            if self.options['polarity'] == 'common-anode':
                # Invert all data lines if a common anode display is used.
                if self.have_dp:
                    oldpins = tuple((1 - state for state in oldpins))
                else:
                    oldpins = tuple((1 - state for state in oldpins[:7]))

            # Convert to character string.
            digit = self.pins_to_hex(oldpins[:7])

            if digit is not None:
                dp = oldpins[7]

                # Check if decimal point is present and active.
                if self.have_dp and dp == 1:
                    digit += '.'

                self.putb(lastpos, self.samplenum, [0, [digit]])

            lastpos = self.samplenum

            oldpins = pins



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Aleksander Alekseev <afiskon@gmail.com>
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.

import sigrokdecode as srd

MAX_DATA_LEN = 128

# Command ID -> name, short description
META = {
    0x00: {'name': 'NOP    ', 'desc': 'No operation'},
    0x01: {'name': 'SWRESET', 'desc': 'Software reset'},
    0x04: {'name': 'RDDID  ', 'desc': 'Read display ID'},
    0x09: {'name': 'RDDST  ', 'desc': 'Read display status'},
    0x10: {'name': 'SLPIN  ', 'desc': 'Sleep in & booster off'},
    0x11: {'name': 'SLPOUT ', 'desc': 'Sleep out & booster on'},
    0x12: {'name': 'PTLON  ', 'desc': 'Partial mode on'},
    0x13: {'name': 'NORON  ', 'desc': 'Partial off (normal)'},
    0x20: {'name': 'INVOFF ', 'desc': 'Display inversion off'},
    0x21: {'name': 'INVON  ', 'desc': 'Display inversion on'},
    0x28: {'name': 'DISPOFF', 'desc': 'Display off'},
    0x29: {'name': 'DISPON ', 'desc': 'Display on'},
    0x2A: {'name': 'CASET  ', 'desc': 'Column address set'},
    0x2B: {'name': 'RASET  ', 'desc': 'Row address set'},
    0x2C: {'name': 'RAMWR  ', 'desc': 'Memory write'},
    0x2E: {'name': 'RAMRD  ', 'desc': 'Memory read'},
    0x30: {'name': 'PTLAR  ', 'desc': 'Partial start/end address set'},
    0x36: {'name': 'MADCTL ', 'desc': 'Memory data address control'},
    0x3A: {'name': 'COLMOD ', 'desc': 'Interface pixel format'},
    0xB1: {'name': 'FRMCTR1', 'desc': 'Frame rate control (in normal mode / full colors)'},
    0xB2: {'name': 'FRMCTR2', 'desc': 'Frame rate control (in idle mode / 8-colors)'},
    0xB3: {'name': 'FRMCTR3', 'desc': 'Frame rate control (in partial mode / full colors) '},
    0xB4: {'name': 'INVCTR ', 'desc': 'Display inversion control'},
    0xB6: {'name': 'DISSET5', 'desc': 'Display function set 5'},
    0xC0: {'name': 'PWCTR1 ', 'desc': 'Power control 1'},
    0xC1: {'name': 'PWCTR2 ', 'desc': 'Power control 2'},
    0xC2: {'name': 'PWCTR3 ', 'desc': 'Power control 3'},
    0xC3: {'name': 'PWCTR4 ', 'desc': 'Power control 4'},
    0xC4: {'name': 'PWCTR5 ', 'desc': 'Power control 5'},
    0xC5: {'name': 'VMCTR1 ', 'desc': 'VCOM control 1'},
    0xDA: {'name': 'RDID1  ', 'desc': 'Read ID1'},
    0xDB: {'name': 'RDID2  ', 'desc': 'Read ID2'},
    0xDC: {'name': 'RDID3  ', 'desc': 'Read ID3'},
    0xDD: {'name': 'RDID4  ', 'desc': 'Read ID4'},
    0xFC: {'name': 'PWCTR6 ', 'desc': 'Power control 6'},
    0xE0: {'name': 'GMCTRP1', 'desc': 'Gamma \'+\'polarity correction characteristics setting'},
    0xE1: {'name': 'GMCTRN1', 'desc': 'Gamma \'-\'polarity correction characteristics setting'},
}

class Ann:
    BITS, CMD, DATA, DESC = range(4)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'st7735'
    name = 'ST7735'
    longname = 'Sitronix ST7735'
    desc = 'Sitronix ST7735 TFT controller protocol.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['Display', 'IC']
    channels = (
        {'id': 'cs', 'name': 'CS#', 'desc': 'Chip-select'},
        {'id': 'clk', 'name': 'CLK', 'desc': 'Clock'},
        {'id': 'mosi', 'name': 'MOSI', 'desc': 'Master out, slave in'},
        {'id': 'dc', 'name': 'DC', 'desc': 'Data or command'}
    )
    annotations = (
        ('bit', 'Bit'),
        ('command', 'Command'),
        ('data', 'Data'),
        ('description', 'Description'),
    )
    annotation_rows = (
        ('bits', 'Bits', (Ann.BITS,)),
        ('fields', 'Fields', (Ann.CMD, Ann.DATA)),
        ('description', 'Description', (Ann.DESC,)),
    )

    def __init__(self):
        self.reset()

    def reset(self):
        self.accum_byte = 0
        self.accum_bits_num = 0
        self.bit_ss = -1
        self.byte_ss = -1
        self.current_bit = -1

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def put_desc(self, ss, es, cmd, data):
        if cmd == -1:
            return
        if META[cmd]:
            self.put(ss, es, self.out_ann, [Ann.DESC,
                ['%s: %s' % (META[cmd]['name'].strip(), META[cmd]['desc'])]])
        else:
            # Default description:
            dots = ''
            if len(data) == MAX_DATA_LEN:
                data = data[:-1]
                dots = '...'
            data_str = '(none)'
            if len(data) > 0:
                data_str = ' '.join(['%02X' % b for b in data])
            self.put(ss, es, self.out_ann, [Ann.DESC,
                ['Unknown command: %02X. Data: %s%s' % (cmd, data_str, dots)]])

    def decode(self):
        current_cmd = -1
        current_data = []
        desc_ss = -1
        desc_es = -1
        self.reset()
        while True:
            # Check data on both CLK edges.
            (cs, clk, mosi, dc) = self.wait({1: 'e'})

            if cs == 1: # Wait for CS = low, ignore the rest.
                self.reset()
                continue

            if clk == 1:
                # Read one bit.
                self.bit_ss = self.samplenum
                if self.accum_bits_num == 0:
                    self.byte_ss = self.samplenum
                self.current_bit = mosi

            if (clk == 0) and (self.current_bit >= 0):
                # Process one bit.
                self.put(self.bit_ss, self.samplenum, self.out_ann,
                         [Ann.BITS, [str(self.current_bit)]])
                self.accum_byte = (self.accum_byte << 1) | self.current_bit # MSB-first.
                self.accum_bits_num += 1
                if self.accum_bits_num == 8:
                    # Process one byte.
                    ann = Ann.DATA if dc else Ann.CMD # DC = low for commands.
                    self.put(self.byte_ss, self.samplenum, self.out_ann,
                             [ann, ['%02X' % self.accum_byte]])
                    if ann == Ann.CMD:
                        self.put_desc(desc_ss, desc_es, current_cmd, current_data)
                        desc_ss = self.byte_ss
                        desc_es = self.samplenum # For cmds without data.
                        current_cmd = self.accum_byte
                        current_data = []
                    else:
                        if len(current_data) < MAX_DATA_LEN:
                            current_data += [self.accum_byte]
                        desc_es = self.samplenum

                    self.accum_bits_num = 0
                    self.accum_byte = 0
                    self.byte_ss = -1
                self.current_bit = -1
                self.bit_ss = -1



##
## This file is part of the libsigrokdecode project.
##
## Copyright (C) 2018 Max Weller
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, see <http://www.gnu.org/licenses/>.
##

import re
import sigrokdecode as srd

ann_cmdbit, ann_databit, ann_cmd, ann_data, ann_warning = range(5)

class Decoder(srd.Decoder):
    api_version = 3
    id = 'sda2506'
    name = 'SDA2506'
    longname = 'Siemens SDA 2506-5'
    desc = 'Serial nonvolatile 1-Kbit EEPROM.'
    license = 'gplv2+'
    inputs = ['logic']
    outputs = []
    tags = ['IC', 'Memory']
    channels = (
        {'id': 'clk', 'name': 'CLK', 'desc': 'Clock'},
        {'id': 'd', 'name': 'DATA', 'desc': 'Data'},
        {'id': 'ce', 'name': 'CE#', 'desc': 'Chip-enable'},
    )
    annotations = (
        ('cmdbit', 'Command bit'),
        ('databit', 'Data bit'),
        ('cmd', 'Command'),
        ('data', 'Data byte'),
        ('warnings', 'Human-readable warnings'),
    )
    annotation_rows = (
        ('bits', 'Bits', (ann_cmdbit, ann_databit)),
        ('data', 'Data', (ann_data,)),
        ('commands', 'Commands', (ann_cmd,)),
        ('warnings', 'Warnings', (ann_warning,)),
    )

    def __init__(self):
        self.samplerate = None
        self.reset()

    def reset(self):
        self.cmdbits = []
        self.databits = []

    def metadata(self, key, value):
        if key == srd.SRD_CONF_SAMPLERATE:
            self.samplerate = value

    def start(self):
        self.out_ann = self.register(srd.OUTPUT_ANN)

    def putbit(self, ss, es, typ, value):
        self.put(ss, es, self.out_ann, [typ, ['%s' % (value)]])

    def putdata(self, ss, es):
        value = 0
        for i in range(8):
            value = (value << 1) | self.databits[i]
        self.put(ss, es, self.out_ann, [ann_data, ['%02X' % (value)]])

    def decode_bits(self, offset, width):
        out = 0
        for i in range(width):
            out = (out << 1) | self.cmdbits[offset + i][0]
        return (out, self.cmdbits[offset + width - 1][1], self.cmdbits[offset][2])

    def decode_field(self, name, offset, width):
        val, ss, es = self.decode_bits(offset, width)
        self.put(ss, es, self.out_ann, [ann_data, ['%s: %02X' % (name, val)]])
        return val

    def decode(self):
        while True:
            # Wait for CLK edge or CE edge.
            clk, d, ce = self.wait([{0: 'e'}, {2: 'e'}])

            if self.matched[0] and ce == 1 and clk == 1:
                # Rising clk edge and command mode.
                bitstart = self.samplenum
                self.wait({0: 'f'})
                self.cmdbits = [(d, bitstart, self.samplenum)] + self.cmdbits
                if len(self.cmdbits) > 24:
                    self.cmdbits = self.cmdbits[0:24]
                self.putbit(bitstart, self.samplenum, ann_cmdbit, d)
            elif self.matched[0] and ce == 0 and clk == 0:
                # Falling clk edge and data mode.
                bitstart = self.samplenum
                clk, d, ce = self.wait([{'skip': int(2.5 * (1e6 / self.samplerate))}, {0: 'r'}, {2: 'e'}]) # Wait 25 us for data ready.
                if self.matched == (True, False, False):
                    self.wait([{0: 'r'}, {2: 'e'}])
                if len(self.databits) == 0:
                    self.datastart = bitstart
                self.databits = [d] + self.databits
                self.putbit(bitstart, self.samplenum, ann_databit, d)
                if len(self.databits) == 8:
                    self.putdata(self.datastart, self.samplenum)
                    self.databits = []
            elif self.matched[1] and ce == 0:
                # Chip enable edge.
                try:
                    self.decode_field('addr', 1, 7)
                    self.decode_field('CB', 0, 1)
                    if self.cmdbits[0][0] == 0:
                        # Beginning read command.
                        self.decode_field('read', 1, 7)
                        self.put(self.cmdbits[7][1], self.samplenum,
                            self.out_ann, [ann_cmd, ['read' ]])
                    elif d == 0:
                        # Beginning write command.
                        self.decode_field('data', 8, 8)
                        addr, ss, es = self.decode_bits(1, 7)
                        data, ss, es = self.decode_bits(8, 8)
                        cmdstart = self.samplenum
                        self.wait({2: 'r'})
                        self.put(cmdstart, self.samplenum, self.out_ann,
                            [ann_cmd, ['Write to %02X: %02X' % (addr, data)]])
                    else:
                        # Beginning erase command.
                        val, ss, es = self.decode_bits(1, 7)
                        cmdstart = self.samplenum
                        self.wait({2: 'r'})
                        self.put(cmdstart, self.samplenum, self.out_ann,
                            [ann_cmd, ['Erase: %02X' % (val)]])
                    self.databits = []
                except Exception as ex:
                    self.reset()
